header {package edu.vub.crime.parser;}

class ParserImpl extends Parser;

options {
  k = 2;
  buildAST = true;
  defaultErrorHandler = false;
}

// CRIME Code consists of a list of facts and rules
program!
	: { #program = #([AGPROG, "program"]); } (s:statement { #program.addChild(#s); })* EOF
	;
	
statement!
	: /* package declaration */
	  "package" pack:indexed_name
	  	{	#statement = #([AGPACK, "package"], #pack); }
	| /* action definition */
	  "action" abbrev:action_name "is" full:action_name
		{	#statement = #([AGABBR, "action"], #abbrev, #full); }
	| /* type definition */
	  "type" t:type_name "extends" parents:type_names
		{	#statement = #([AGEXTS, "extends"], #t, #parents); }	
	| exp:logic_expression
		{	#statement = #exp; }
	;
	
logic_expression!
	: (COLON) => act:action r:rule[#act]
		{	#logic_expression = #r; }
	| f:fact exp:assert_or_rule[#f]
		{	#logic_expression = #exp; }
	;
	
fact
	: label:pattern_name optional_attributes[#label]
		{	#fact = #label; }
	;
	
assert_or_rule![AST aFact] 
	: DOT 
		{ #assert_or_rule = #([AGASRT, "assert"], #aFact); }
	| r:rule[#aFact]
		{ #assert_or_rule = #r; }
	;
	
rule![AST head]
	: more_effects[#head]
	  INFERS body:pattern_list
	  	{	#rule = #([AGRULE, "rule"], #head, #([AGCOND, "condition"], body)); }
	;

more_effects![AST previousEffect]
	: COMMA nextEffect:effect
		{	#previousEffect.setNextSibling(#nextEffect); }
	  more_effects[#nextEffect]
	| 
	;
	
effect
	: action
	| fact
	;
	

action!
{ AST this_action = #([AGEVNT, "event"]); }
	: act:action_name 
		{	#act.setText("name");
			#this_action.addChild(#act); }

	  optional_attributes[#this_action]

	  	{	#action = #this_action; }
	;		
	
pattern_name!
	: sym:type_name posq:possible_qualification[#sym] 
		{	#pattern_name = #posq; }
	;
						
possible_qualification![AST sym]
	: ARROW nam:type_name 
		{	#sym.setText("space");
			#nam.setText("name");
			#possible_qualification = #([AGQUA, "public fact"], #sym, #nam); }
	| 	{ 	#sym.setText("name");
			#possible_qualification = #([AGUNQ, "private fact"], #sym); }
	;

optional_attributes![AST parent]
	: LPR RPR
	| LPR named_attr:pair more_pairs[#named_attr]
		{	#parent.addChild(#named_attr); }
	;

pattern_list!
	: pat:pattern pats:more_patterns[#pat]
		{	#pattern_list = #pats; }
	;

more_patterns![AST pat]
	: COMMA patl:pattern_list
		{	#pat.setNextSibling(#patl); 
			#more_patterns = #pat; }
	| DOT
		{	#more_patterns = #pat; }
	| (RPR) =>
		{	#more_patterns = #pat; }
	;

pattern!
	: "findall" fnd:accumulator_expression
		{	#fnd.setType(AGFNDA); #fnd.setText("findall"); #pattern = #fnd; }
		
	| "bagof"	bag:accumulator_expression				
		{	#bag.setType(AGBGOF); #bag.setText("findall"); #pattern = #bag; }
				
	| "length" LPR varlist:attribute COMMA varlength:attribute RPR 
		{	#pattern = #([AGLEN,"length"], #varlist, #varlength); }	
							
	| "not" negated_pattern:fact
		{	#pattern = #([AGNOT,"not"], #negated_pattern); }	
 				
	| simple_pattern:fact
		{	#pattern = #simple_pattern;} 				
	
	| var:variable relop:RELATIONAL_OPERATOR val:attribute
		{	#pattern = #([AGXCON,"constraint"], #var, #([AGCMP,"operator"],#relop), #val); }
	;	

accumulator_expression!
	: LPR vars:attribute COMMA  LBR query:pattern_list RBR COMMA cumulative:attribute RPR
		{  #accumulator_expression = #([AGNULL, "placeholder"], #vars, #query, cumulative); }
	;

attribute!	
	: val:value 	{ #attribute = #val; }
	| var:variable	{ #attribute = #var; }
	| lis:list		{ #attribute = #lis; }
	| obj:object	{ #attribute = #obj; }
	;
			
list!
	: LBR attr:attribute more_list_elements[#attr]
		{	#list = #([AGLST,"list"], #attr); }
	| LBR RBR 
		{	#list = #([AGNIL,"null"], #([AGLST,"list"])); }			
	;
			 
more_list_elements![AST previousElement]
	: COMMA nextElement:attribute more_list_elements[#nextElement]
		{	#previousElement.setNextSibling(#nextElement); }
	| RBR
	| PIP cdr:attribute RBR
		{	#previousElement.setNextSibling(#([AGTAIL,"tail"], #cdr)); }
	;
	
value
	:! nbr:NBR	{ #value = #([AGNBR,"number"],nbr); }
	|! frc:FRC	{ #value = #([AGFRC,"fraction"],frc); }
	|! txt:TXT	{ #value = #([AGTXT,"text"],txt); }
	| symbol
	| type_name
	;
         
object!
	: name:type_name LPR pair:pair morepr:more_pairs[#pair]
		{
			#object = #([AGOBJ,"object"], #name, #pair);
		}
	;

pair! 
	: name:simple_name op:RELATIONAL_OPERATOR value:attribute
		{
			#pair =  #([AGPAIR,"pair"], #name, #([AGCMP,"operator"], #op), #value);
		}
	;

more_pairs![AST pair]	
	: COMMA opair:pair morepr:more_pairs[#opair]
		{
			#pair.setNextSibling(#morepr);
			#more_pairs = #pair;
		}
	| RPR 
		{	#more_pairs = #pair; }
	;
	
type_name!
	: name:NAM
		{	#type_name = #([AGTYPE,"type"], #name); }
	;
	
type_names!
	: t:type_name more_types[#t]
		{	#type_names = #t; }
	;
	
more_types![AST previousType] 
	: (COMMA) => 
		  COMMA nextType:type_name
			{	#previousType.setNextSibling(#nextType); }
		  more_types[#nextType]
	|
	;
			
variable!
	: QUESTION nam:indexed_name
		{	#variable = #([AGVAR, "variable"], #nam); }
	;
	
action_name!
	: COLON nam:indexed_name
		{ 	#action_name = #([AGEVNT, "event"], #nam); }
	;
	
indexed_name!
	: n:NAM indices[#n]
		{	#indexed_name = #n; }
	;
	
indices![AST previousIndex] 
	: (DOT) => 
		  DOT nextIndex:NAM
			{	#previousIndex.setNextSibling(#nextIndex); }
		  indices[#nextIndex]
	|
	;			

symbol!
	: BQU var:NAM { #symbol = #([AGSYM,"symbol"], var); };

simple_name!
	: n:NAM
		{ #simple_name = #([AGNAME, "name"], #n); }
	;



//----------------------------------------------------------------------------------------------
//
//  LEXER
//
//----------------------------------------------------------------------------------------------
class LexerImpl extends Lexer;

options {
  k = 3;
}

// Protected Scanner Tokens

// OUTPUT TOKENS
// These tokens are never produced by the scanner itself as they are protected. 
// However they are used to annotate the resulting ANTLR tree, so that the walker
// can easily produce the correct Java ATAbstractGrammar elements.
// Each token definition aligns the token with its printed representation.
// Statements
protected AGPROG	: "top-level";
protected AGPACK	: "package";
protected AGABBR	: "action";
protected AGTYPE	: "type";
protected AGEXTS	: "extends";
protected AGNOT		: "not";
protected AGNULL	: "placeholder"; // for a findall of bagof accumulator node 
protected AGFNDA	: "findall";
protected AGBGOF	: "bagof";
protected AGLEN 	: "length";
protected AGASRT	: "assert";
protected AGRULE	: "rule";
protected AGCOND	: "condition";
protected AGXCON	: "constraint";

// Naming nodes for actions, facts and patterns
protected AGQUA		: "spaceName -> factName";
protected AGUNQ		: "factName";
protected AGEVNT	: ":eventName";

// Compound Attributes
protected AGLST		: "list";	
protected AGNIL		: "nilist";
protected AGTAIL	: "tail";	
protected AGOBJ		: "object";
protected AGPAIR	: "pair";
protected AGNAME	: "name";


// Literals
protected AGVAR		: "variable";
protected AGNBR		: "number";
protected AGFRC		: "fraction";
protected AGTXT		: "text";
protected AGSYM		: "symbol";

// auxiliary tokens for operators
protected AGCMP     : "symbol";
protected AGADD     : "symbol";
protected AGMUL     : "symbol";
protected AGPOW     : "symbol";

protected DIGIT: '0'..'9'
    ;
    
protected LETTER: ('a'..'z'|'A'..'Z' | '_' )
    ;

protected EXPONENT: ('e' | 'E')
	;
	
protected CMPCHAR: ('!' | '<' | '=' | '>' | '~' )
	;

QUESTION: '?'
	;
		
protected SIGN: ('+' | '-' )
	;
	
protected SCALE: EXPONENT (SIGN)? NBR
	;
	
DOT: '.'
	; 

COMMA: ','
	;	
	
ARROW: "->"
	;
	
PIP: "|"
	;	
	
BQU: "`"
	;
		
protected COLON: ':'
	;
	
protected INFERS: ":-"
	;

COLON_OR_INFERS
	: (COLON '-') => INFERS 		{ $setType(INFERS); }
	| COLON							{ $setType(COLON);  }
	;	    
	
RELATIONAL_OPERATOR: ( '=' | "!=" | "<=" | ">=" | '<' | '>' )
	;
	
protected NBR: (DIGIT)+
	;

protected FRC: NBR (SCALE | DOT NBR (SCALE)?)
	;
	
NBR_OR_FRC options { paraphrase = "a number or fraction"; }: 
			( NBR EXPONENT ) => FRC  { $setType(FRC); }
          | ( NBR DOT NBR) => FRC    { $setType(FRC); }
          |   NBR                    { $setType(NBR); }
    ;

protected CMP: CMPCHAR (CMPCHAR)?
	;

NAM options { paraphrase = "a symbol"; }: LETTER (DIGIT | LETTER)*
	;
	
WHITESPACE options { paraphrase = "whitespace"; }: ('\t' |  ' ')
           { $setType(Token.SKIP); }
    ;
    
NEWLINE options { paraphrase = "newline"; }:  ( "\r\n" | '\r' | '\n')
          { newline(); 
            $setType(Token.SKIP); }
    ;
    
LPR options { paraphrase = "a left parenthesis"; }: '(';
RPR options { paraphrase = "a right parenthesis"; }: ')';

LBR options { paraphrase = "a left bracket"; }: '[';
RBR options { paraphrase = "a right bracket"; }: ']';

TXT options { paraphrase = "a text string"; }: '"' (ESC|~('"'|'\\'|'\n'|'\r'))* '"'
	;

// Single-line comments
SL_COMMENT options { paraphrase = "a single-line comment"; }
	:	"//" WHITESPACE
		(~('\n'|'\r'))* ('\n'|'\r'('\n')?)
		{$setType(Token.SKIP); newline();}
	;

// Multiple-line comments
ML_COMMENT options { paraphrase = "a multi-line comment"; }
	:	"/*"
		(	/*	'\r' '\n' can be matched in one alternative or by matching
				'\r' in one iteration and '\n' in another. I am trying to
				handle any flavor of newline that comes in, but the language
				that allows both "\r\n" and "\r" and "\n" to all be valid
				newline is ambiguous. Consequently, the resulting grammar
				must be ambiguous. I'm shutting this warning off.
			 */
			options {
				generateAmbigWarnings=false;
			}
		:
			{ LA(2)!='/' }? '*'
		|	'\r''\n'		{newline();}
		|	'\r'			{newline();}
		|	'\n'			{newline();}
		|	~('*'|'\n'|'\r')
		)*
		"*/"
		{$setType(Token.SKIP);}
	;
	
protected ESC
	:	'\\'
		(	'n'
		|	'r'
		|	't'
		|	'b'
		|	'f'
		|	'"'
		|	'\''
		|	'\\'
		|	'0'..'3'
			(
				options {
					warnWhenFollowAmbig = false;
				}
			:	'0'..'7'
				(
					options {
						warnWhenFollowAmbig = false;
					}
				:	'0'..'7'
				)?
			)?
		|	'4'..'7'
			(
				options {
					warnWhenFollowAmbig = false;
				}
			:	'0'..'7'
			)?
		)
	;


//----------------------------------------------------------------------------------------------
//
//  WALKER
//
//----------------------------------------------------------------------------------------------

{	
	import edu.vub.crime.grammar.*;
	import edu.vub.crime.grammar.attributes.*;
	import edu.vub.crime.grammar.attributes.constant.*;
	import edu.vub.crime.grammar.attributes.datastructures.*;

	import java.util.Vector;
}
class TreeWalkerImpl extends TreeParser;

program returns [ ExpressionList expressions ] throws CrimeParseException
	{
		expressions = new ExpressionList();
	}
		: #(AGPROG ( statement_or_expression[expressions] ) *)
		;
		
statement_or_expression[ ExpressionList expressions ] throws CrimeParseException
	{
		String package_name, short_name, full_name, child, aParent;
		Vector allParents = new Vector();
		Expression expr;
	}
		: #(AGPACK package_name=indexed_name) 
			{	ExpressionFactory.setPackage(package_name); }
			
		| #(AGABBR short_name=indexed_name full_name=indexed_name) 
			{	ExpressionFactory.registerAbbreviation(short_name, full_name); }
			
		| #(AGEXTS child = type_name 
			(aParent = type_name 
				{	allParents.add(ExpressionFactory.type(aParent)); }
			)+ 
		)
			{
				Type[] theParents = (Type[]) allParents.toArray(new Type[ allParents.size() ]);
				ExpressionFactory.registerSubtypeRelationship(child, theParents);
			}
			
		| expr = expression { expressions.add( expr ); }
		;

expression returns [ Expression expr ] throws CrimeParseException
	{	expr = null; }
		: expr=assertion
		| expr=rule
		;

assertion returns [ Expression expr ] throws CrimeParseException
	{	expr = null; }
	
		: #(AGASRT expr=fact)
			{	if(! expr.variables().isEmpty())
					throw new CrimeParseException("Asserted facts may never contain variables");
			}
		;

rule returns [ Expression expr ] throws CrimeParseException
	{
		expr = null;
		ExpressionList effects;
		ExpressionList prerequisites; 
	}
			: #(AGRULE effects=effect_list #(AGCOND prerequisites=pattern_list))
			
			  {				  				  	
			  	if( prerequisites.variables().containsAll( effects.variables() )) {
			  		RuleConsequenceVisitor theVisitor = new RuleConsequenceVisitor();
			  		
			  		effects.accept(theVisitor);

			  		expr = ExpressionFactory.rule(theVisitor.getEvents(), prerequisites);
			  	} else {
			  		//Warning effect variables are mutated !!
			  		effects.variables().removeAll(prerequisites.variables());
			  		throw new CrimeParseException("Effect of a rule contains unbound variables "+ effects.variables());
			  	}
			  }
			;

effect_list returns [ExpressionList effects]
	{	
					effects = new ExpressionList();
		Expression	expr  = null;
	}
			: ( expr = effect { effects.add(expr); }) +
			;

effect returns [ AttributedExpression expr ]
	{	
		expr = null;
		String name; 
	}
			: expr=fact
			| #(AGEVNT #(AGEVNT name=indexed_name)
					{	expr = ExpressionFactory.event(name); }
				attributes[expr])
			;

pattern_list returns [ ExpressionList patterns ] throws CrimeParseException
	{	
		patterns = new ExpressionList(); 
		Expression expr;
	}
			: ( expr=pattern { if(expr instanceof Property)
												patterns.addProperty( (Property)expr );
											else
												patterns.add( expr ); })*
			;
			

pattern returns [ Expression pattern ] throws CrimeParseException
	{	
		pattern = null;
		
		GenericFact		negatedFact;
		
		Variable 		accumulatedVar, 
						collectionVar;
		ExpressionList	query;
		
		Attribute		listAttribute,
						lengthAttribute;
	}
			: pattern=fact

			| #(AGNOT negatedFact=fact)
				{	pattern = ExpressionFactory.negation(negatedFact);	}

			| #(AGFNDA accumulatedVar=simple_variable query=pattern_list collectionVar=simple_variable)
				{	pattern = ExpressionFactory.findAll(accumulatedVar, query, collectionVar); }
				
			| #(AGBGOF accumulatedVar=simple_variable query=pattern_list collectionVar=simple_variable)
				{	pattern = ExpressionFactory.bagOf(accumulatedVar, query, collectionVar); }
				
			| #(AGLEN  listAttribute=attribute lengthAttribute=attribute)
				{	pattern = ExpressionFactory.length(listAttribute,lengthAttribute); }
				
//			| pattern=constraint
			;

/* / ?dpi > 500, ?printer.dpi > 500, ?office.printer.owner = `PROG, ...
constraint returns [ Expression expr ]
	{	
		String 		theVariableName;
		Operator	theComparison;
		Attribute	theExpectedValue;
		LinkedList	indexList = new LinkedList();
	} 
			: #(AGXCON 
				#(AGVAR 
					(first:NAM { theVariableName = first.getText(); }) 
					(index:NAM { indexList.add(index.getText()); })* )
				#(AGCMP op:RELATIONAL_OPERATOR) { theComparison = Operator.getOperator(op.getText()); }
				theExpectedValue = attribute
			)
				{
					int theNumberOfAdditionalIndices = indexList.size();
					
					if(theNumberOfAdditionalIndices != 0) {
						String[]	theIndices = (String[]) indexList.toArray(new String[theNumberOfAdditionalIndices]);
						
						for(int i = theNumberOfAdditionalIndices - 1; i < 0; i--) {
							String 	theIndex = theIndices[i];
							
							Pair 	theField = ExpressionFactory.pair(theIndex, theComparison, theExpectedValue);
							theExpectedValue = ExpressionFactory.object();
							theComparison	 = Operator.EQL;
							
							((TypedObject) theExpectedValue).addField(theField);
						}
					}
					
					expr = ExpressionFactory.property(theVariableName, theComparison, theExpectedValue);
				}
			; */

fact returns [ GenericFact aFact ]
	{	
		aFact = null;
		String name, space;
	}
			: #(AGQUA space=type_name name=type_name 
					{	aFact = ExpressionFactory.qualifiedFact(space, name); }
				attributes [ aFact ])	
			| #(AGUNQ name=type_name 
					{	aFact = ExpressionFactory.unqualifiedFact(name); }
				attributes[aFact])		
			;

		
attributes[ AttributedExpression fact_or_event ]
	{
		Pair theAttribute;
		IndexingVisitor indx = new IndexingVisitor(fact_or_event);
	}
			: (theAttribute = pair { indx.nextAttribute(theAttribute.getName(), theAttribute.getValue()); })* 
			;

attribute returns [ Attribute attr ]
	{	
		attr = null;
	}
			: attr=literal
			| attr=simple_variable
			| attr=list
			| attr=object
			;
			
object returns [ TypedObject theObject ] 
	{
					theObject = null;
		Attribute 	theType;
		Pair 		theField;
	}
		: #(AGOBJ 
			theType = literal
				{	theObject = ExpressionFactory.object(theType); }
			
			(theField = pair 
			 	{	theObject.addField ( theField ); } 
			)* )
		;
		
pair returns [ Pair thePair ]
	{
					thePair			= null;
		String		theFieldName	= null;
		Operator 	theOperator		= null;
		Attribute	theFieldValue	= null;
	}
		: #(AGPAIR theFieldName=name #(AGCMP op:RELATIONAL_OPERATOR) theFieldValue = attribute)
			{	theOperator = Operator.getOperator(op.getText());
				// 
				if(theOperator != Operator.EQL && theOperator != Operator.NEQ)
					if(	!(theFieldValue instanceof edu.vub.crime.grammar.attributes.constant.Number) && 
						!(theFieldValue instanceof edu.vub.crime.grammar.attributes.constant.Fraction) && 
						!(theFieldValue instanceof edu.vub.crime.grammar.attributes.Variable))
						throw new RuntimeException("Cannot compare non-numeric attributes other than for (non-)equality");
				thePair = ExpressionFactory.pair(theFieldName, theOperator, theFieldValue);
			}
		;
	
list returns [ List theList ]
	{
					theList = ExpressionFactory.list();
		Attribute	theListElement;
		Attribute	theListTail;
	}
		: #(AGLST 
				(theListElement = attribute
					{	theList.add( theListElement ); } )+ 
				(#(AGTAIL (theListTail = attribute 
					{	theList.setTail( theListTail ); } )))? )
				
		| #(AGNIL AGLST)
		;
			
literal returns [ Attribute lit]
	{
		lit = null;
  	}
			: #(AGNBR nbr:NBR) { lit = ExpressionFactory.attribute(Integer.parseInt(nbr.getText())); }
			| #(AGFRC frc:FRC) { lit = ExpressionFactory.attribute(Double.parseDouble(frc.getText())); }
			| #(AGTXT txt:TXT) { String text = txt.getText(); lit = ExpressionFactory.attribute(text.substring(1, text.length() - 1)); }
			| #(AGSYM sym:NAM) { lit = ExpressionFactory.symbol(sym.getText()); }
			| {	String type; }			
      			type=type_name      			
      		  { lit = ExpressionFactory.type(type); }
          ;
          
type_name returns [ String type ]
	{	
		type = null; 
	}
          : #(AGTYPE txt:NAM) { type = txt.getText(); }
          ;
          
indexed_name returns [ String theName ]
	{
			 			theName 		= null;
		StringBuffer 	theFullName		= new StringBuffer();
	}
		: (options {greedy=true;} : next:NAM
				{	String 	theConstituent	= next.getText();
					
					if(theFullName.length() != 0)
						theFullName.append(".");
					
					theFullName.append(theConstituent);
				} )+
			
			{	theName = theFullName.toString(); }
		;
          
simple_variable returns [ Variable var ]
	{	var = null; }
          : #(AGVAR txt:NAM) { var = ExpressionFactory.variable(txt.getText()); }
          ;
          
name returns [ String n ]
	{	
		n = null; 
	}
		: #(AGNAME txt:NAM) { n = txt.getText(); }
		;