package edu.vub.crime;


import java.util.Iterator;

import edu.vub.crime.grammar.ExpressionFactory;
import edu.vub.crime.grammar.GenericFact;
import edu.vub.crime.grammar.attributes.constant.Type;
import edu.vub.crime.grammar.attributes.datastructures.Pair;
import edu.vub.crime.grammar.attributes.datastructures.TypedObject;

/** <pre>
 * This class is used for asserting and retracting a fact of the engine.
 * This class is a built-in action, which handles the adding of a fact
 * to the network (when the rule from which this action is part of its
 * left-hand-side is triggered), or the removal of that fact 
 * (when the conditions of that rule no longer met).
 * </pre> */
public class Assert extends Action {

	/** <pre>
	 * This method will cause the assertion of a new
	 * fact to the engine.  This fact can be built, because
	 * the arguments of that fact are passed as an argument.
	 * </pre> */
	public void activated(TypedObject arguments) {
		createFact(arguments).assert(itsEngine);
	}

	/** <pre>
	 * This method causes the retracting of a fact in the engine.
	 * </pre> */
	public void deactivate(TypedObject arguments) {		
		createFact(arguments).retract(itsEngine);
	}

	/** <pre>
	 * A new fact is created, given its arguments.
	 * </pre> */
	private GenericFact createFact(TypedObject arguments) {
		Type name = (Type) arguments.getType();		
		GenericFact fact = ExpressionFactory.unqualifiedFact(name.toString());
		
		for (Iterator attributeIt = arguments.iterator(); attributeIt.hasNext();) {
			Pair currentAttribute = (Pair) attributeIt.next();
			
			fact.addNamedAttribute(currentAttribute.getName(), currentAttribute.getValue());
		}

		return fact;
	}
}