package edu.vub.crime;

import edu.vub.crime.grammar.Rule;
import edu.vub.crime.rete.CompileNetworkVisitor;

public interface RuleBase {

	/** 
	 * Deploys a rule, which implies compiling it so that its prerequisites are compiled
	 * to a rete network where each fact is compiled to a filter node which are joined
	 * one by one using join nodes. The consequence part of the rule is then compiled to
	 * a single production node which is triggered by the rete network representing the 
	 * prerequisites of the rule.
	 * <p>
	 * The compilation of a rule is performed by the {@link CompileNetworkVisitor}. 
	 */
	public abstract void deploy(Rule rule);

	/** <pre>
	 * A future request, not yet implemented,
	 * the general idea is to remove the rule but also
	 * remove all it's derivations. 
	 * The later is not that easy for the moment.
	 * </pre> */
	public abstract void undeploy(Rule theRule);

}