package edu.vub.crime;

import edu.vub.crime.grammar.GenericFact;
import edu.vub.crime.grammar.Rule;
import edu.vub.crime.rete.CompileNetworkVisitor;
import edu.vub.crime.rete.nodes.FilterNode;
import edu.vub.crime.rete.nodes.ViewNode;

public class View implements RuleBase {

	protected CrimeEngine	itsEngine;
	protected ViewNode 		itsRootNode;
	
	public View(CrimeEngine theEngine) {
		itsEngine 	= theEngine;
		itsRootNode	= new ViewNode(theEngine.itsRootNode);
	}
	
	public void deploy(Rule rule) {
		rule.accept(new CompileNetworkVisitor(itsEngine, itsRootNode));
	}

	public void undeploy(Rule theRule) {
		itsRootNode.undeploy(theRule);
	}
	
	public void allow(GenericFact theFact) {
		itsRootNode.restrict(theFact.getLabel());
		
		// Create a compiler visitor which will be used to compile the Fact into a filter node
		CompileNetworkVisitor theCompiler = new CompileNetworkVisitor(itsEngine, itsEngine.itsRootNode);
		
		// Let the compiler traverse the Fact
		theFact.accept(theCompiler);
		
		// The result is a new filter node which checks whether published facts have matching constants etc.
		FilterNode theNewFilter = (FilterNode) theCompiler.getCompiledNode();
				
		theNewFilter.addChild(itsRootNode);
	}
}
