package edu.vub.crime.dataflow;

import java.util.Iterator;

import edu.vub.crime.CrimeEngine;
import edu.vub.crime.rete.agenda.Activation;
import edu.vub.crime.rete.data.DataFactory;
import edu.vub.crime.rete.data.Token;
import edu.vub.crime.rete.data.TokenTag;
import edu.vub.crime.rete.nodes.FilterNode;

/**
 * This activation class is scheduled when the current value of a dataflow attribute that is used 
 * in a filter node has changed. Executing it will prompt the filter node to reevaluate itself.
 * 
 * @author smostinc
 *
 */
public class FilterReevaluation extends Activation {

	private static final Token DATAFLOW_Token = DataFactory.token(TokenTag.unconditional);
	
	private FilterNode itsNodeToTrigger;
	
	public FilterReevaluation(FilterNode theNodeToReevaluate) {
		super(DATAFLOW_Token);
		
		itsNodeToTrigger = theNodeToReevaluate;
	}
	
	public void execute(CrimeEngine engine) {
		itsNodeToTrigger.reevaluate();
	}

	public boolean isComplementOf(Activation possibleComplement) {
		return false;
	}

	public Iterator iterator() {
		throw new RuntimeException("Not supported");
	}

}
