package edu.vub.crime.grammar;

import java.util.Stack;

import edu.vub.crime.grammar.attributes.Attribute;
import edu.vub.crime.grammar.attributes.Variable;

public abstract class AccumulatingPrimitive extends Primitive {

	/** <pre>
	 * var_ : is an attribute 
	 *        In the example, the var_ equals the variable ?person.
	 * query_ : list containing the queries
	 *          In the example, this list contains the facts
	 *          isPresent(?person, LivingRoom) and
	 *          prefers(?person, ?genre). 
	 * accumulator_ : is a variable
	 *                In the example, accumulator is the variable
	 *                ?similarTastes.  
	 * </pre> */
	protected Attribute itsAccumulatedVariable;
	protected ExpressionList itsQuery;
	protected Variable itsAccumulator;

	public Variable getAccumulator() {
		return itsAccumulator;
	}

	public Expression getQuery() {
		return itsQuery;
	}

	public Attribute getAccumulatedVariable() {
		return itsAccumulatedVariable;
	}
	
	public void initialize(Attribute theAccumulatedVariable, ExpressionList theQuery, Variable theAccumulator){
		if(	itsQuery != null ||
			itsAccumulator != null ||
			itsAccumulatedVariable != null)
			throw new RuntimeException("Attempting to reinitialize an accumulating primitive");
		
		itsAccumulatedVariable = theAccumulatedVariable;
		itsQuery = theQuery;
		itsAccumulator = theAccumulator;
		
		// Since the accumulator is not visited by the indexing visitor, set its index manually 
		Stack accumulatorIndex = new Stack();
		accumulatorIndex.push("accumulated");
		
		theAccumulator.setIndex(accumulatorIndex);
				
	}
}
