/**
 * 
 */
package edu.vub.crime.grammar;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import edu.vub.crime.exceptions.IncorrectAttributeIndexException;
import edu.vub.crime.grammar.attributes.Attribute;
import edu.vub.crime.grammar.attributes.CompoundAttribute;
import edu.vub.crime.grammar.attributes.Variable;
import edu.vub.crime.grammar.attributes.datastructures.TypedObject;

/**
 * Shared parent of both facts and events, which are both expressions which may have an arbitrary number of 
 * named attributes associated with them.
 * 
 * @author smostinc
 */
public abstract class AttributedExpression extends Expression {

	protected final Set 		itsVariables = new HashSet();
	protected final	TypedObject	itsRepresentation;
	
	public AttributedExpression(String itsName) {
		itsRepresentation = ExpressionFactory.object(ExpressionFactory.type(itsName));
	}
	
	protected AttributedExpression(TypedObject anExistingObject) {
		itsRepresentation = anExistingObject;
	}
	
	// ATTRIBUTES
	
	public void addNamedAttribute(String name, Attribute theAttribute) {
		if(theAttribute instanceof Variable) {
			itsVariables.add((Variable)theAttribute);
		} else if(theAttribute instanceof CompoundAttribute) {
			CompoundAttribute theCompoundAttribute = (CompoundAttribute) theAttribute;
			
			itsVariables.addAll(theCompoundAttribute.getVariables());
		}
		
		itsRepresentation.addField(ExpressionFactory.pair(name, theAttribute));
	}
	
	public TypedObject getRepresentation() {
		return itsRepresentation;
	}
	
	/**
	 * Returns the Attribute associated with a (possibly multidimensional) index. Due to the 
	 * introduction of data structures, the index of an attribute is no longer a simple name 
	 * identifying a position in the expression.  Instead, it is a multidimensional index; a 
	 * Vector. 
	 * <p>
	 * This method takes an iterator for this index vector which ensures that <tt>next()</tt> 
	 * returns the index of the attribute in the expression.  Having resolved this index, two
	 * situations might occur:  either the Iterator has more elements, allowing it to further
	 * navigate in a compound attribute.  Alternatively, the Iterator has no more elements in 
	 * which case the attribute can be returned.
	 * 
	 * @param theAttributeIndex an iterator across a (possibly multidimensional) attribute index.
	 */
	public Attribute getAttribute(Iterator theAttributeIndex) throws IncorrectAttributeIndexException {
		return itsRepresentation.getAttribute(theAttributeIndex);
	}
	
	public Iterator attributeIterator() {
		return itsRepresentation.iterator();
	}
	
	public Set variables() {
		return itsVariables;
	}
}
