package edu.vub.crime.grammar;

import java.util.Iterator;

import edu.vub.crime.Action;
import edu.vub.crime.CrimeEngine;
import edu.vub.crime.grammar.attributes.Attribute;
import edu.vub.crime.grammar.attributes.datastructures.Pair;
import edu.vub.crime.grammar.attributes.datastructures.TypedObject;
import edu.vub.crime.grammar.utils.ActivationBuilder;
import edu.vub.crime.grammar.utils.UtilsFactory;
import edu.vub.crime.rete.data.Token;
import edu.vub.crime.rete.data.TokenTag;
import edu.vub.crime.util.Logging;

/** 
 * An event is a grammar element which encapsulates an action that needs to be executed
 * when a given set of prerequisites are met. The Event can be initialized with a given
 * Action instance, or the action can be loaded at startup time using a {@link LiteralEvent}.
 * 
 * @author smostinc
 */	
public class Event extends AttributedExpression {
	protected ActivationBuilder	factBuilder_;
	protected Action			action_;
	protected String			kind_;
		
	// Protected constructor for the LiteralEvent subclass
	protected Event(String actionName) {
		super(actionName);
		
		kind_ = actionName;
	}
	
	public Event(Action theAction) {
		this(theAction.getClass().getName());
		
		action_ = theAction;
	}
	
	public Event(Action theAction, String theKindOfEvent) {
		this(theKindOfEvent);
		
		action_ = theAction;
	}

	public void initialize(CrimeEngine itsEngine, ExpressionList expressions) { 
		ActivationBuilder factBuilder = UtilsFactory.factBuilder(kind_);
		for(Iterator attributeIt = itsRepresentation.iterator(); attributeIt.hasNext();) {
			Pair		currentAttribute	= (Pair) attributeIt.next();

			String		currentName 		= (String) currentAttribute.getName();
			Attribute	currentValue		= (Attribute) currentAttribute.getValue();
			
			factBuilder.addPicker(
					currentName, 
					currentValue.makeAttributePicker(expressions));
		}
		
		setFactBuilder(factBuilder);
		setEngine(itsEngine);
	}
		
	public void produce(Token token){
		TypedObject activation = factBuilder_.buildActivation(token);
		switch(token.getTag()) {
		case TokenTag.add:
			Logging.Engine_LOG.info("activated " + activation);
			action_.activated(activation);
			break;
		case TokenTag.remove:
			Logging.Engine_LOG.info("deactivated " + activation);
			action_.deactivate(activation);
			break;
		}
	}
	
	public void setFactBuilder(ActivationBuilder factBuilder) {
		factBuilder_ = factBuilder;
	}

	public void setEngine(CrimeEngine engine) {
		this.action_.initializeEngine(engine);
	}
	
	public void accept(GrammarVisitor visitor) {
		visitor.visit(this);
	}
	
	public String toString() {
		return itsRepresentation.toString();
	}
}