package edu.vub.crime.grammar;

import java.util.HashMap;

import edu.vub.crime.CrimeEngine;
import edu.vub.crime.grammar.attributes.Attribute;
import edu.vub.crime.grammar.attributes.Variable;
import edu.vub.crime.grammar.attributes.constant.Fraction;
import edu.vub.crime.grammar.attributes.constant.Number;
import edu.vub.crime.grammar.attributes.constant.Symbol;
import edu.vub.crime.grammar.attributes.constant.Text;
import edu.vub.crime.grammar.attributes.constant.Type;
import edu.vub.crime.grammar.attributes.datastructures.List;
import edu.vub.crime.grammar.attributes.datastructures.Operator;
import edu.vub.crime.grammar.attributes.datastructures.Pair;
import edu.vub.crime.grammar.attributes.datastructures.TypedObject;
import edu.vub.crime.grammar.operators.BagOf;
import edu.vub.crime.grammar.operators.FindAll;
import edu.vub.crime.grammar.operators.Length;
import edu.vub.crime.util.Logging;

public class ExpressionFactory {
	
	private static int 			 _VAR_COUNT_ = 0;
	private static final HashMap _ABBR_MAP_  = new HashMap();
	private static final HashMap _TYPE_MAP_  = new HashMap();
	private static final Type	 _ROOT_TYPE_ = new Type("root", new Type[0]);
	
	public static void setPackage(String thePackageName) {
		Logging.Engine_LOG.info("Events will be sought for in package" + thePackageName);
		CrimeEngine.PACKAGE_DECL = thePackageName;
	}
	
	public static void registerAbbreviation(String shortName, String fullName) {
		Logging.Engine_LOG.info("Registering " + shortName + " as an abbreviation for " + fullName);
		_ABBR_MAP_.put(shortName, fullName);
	}
	
	public static String resolveAbbreviation(String shortName) {
		String longName = (String) _ABBR_MAP_.get(shortName);
		
		return longName != null? longName : shortName;
	}
	
	public static void registerSubtypeRelationship(String name, Type[] parents) {
		Logging.Engine_LOG.info("Registering " + name + " as a subtype of the following types " + parents);
		_TYPE_MAP_.put(name, new Type(name, parents));
	}
	
	public static Event event(String name) {
		return new LiteralEvent(name);	
	}
	
	public static GenericFact qualifiedFact(String space, String name) {
		return new GenericFact(space, name);
	}

	public static GenericFact unqualifiedFact(String name) {
		return new GenericFact(GenericFact.PRIVATE, name);
	}
	
	public static Rule rule(ExpressionList consequence, ExpressionList prerequisites) {
		return new Rule(consequence, prerequisites);
	}
	
	public static Attribute attribute(int i) {
		return new Number(i);
	}

	public static Attribute attribute(double d) {
		return new Fraction(d);
	}

	public static Attribute attribute(String s) {
		return new Text(s);
	}

	public static Attribute symbol(String s) {
		return new Symbol(s);
	}

	public static Variable variable(String s) {
		return new Variable(s);
	}
	
	public static Variable variable() {
		return new Variable("gen_var/" + _VAR_COUNT_++);
	}
	
	public static Type type(String name) {
		Type theStoredType = (Type) _TYPE_MAP_.get(name);
		
		if(theStoredType == null) {
			theStoredType = new Type(name, new Type[] { _ROOT_TYPE_ });
			_TYPE_MAP_.put(name, theStoredType);
		}
		return theStoredType;
	}
	
	public static FindAll findAll(Attribute vars, ExpressionList qwery, Variable accum){
		FindAll thePrimitive = new FindAll();
		thePrimitive.initialize(vars, qwery, accum);
		
		return thePrimitive;
	}
	
	public static BagOf bagOf(Attribute vars, ExpressionList qwery, Variable accum){
		BagOf thePrimitive = new BagOf();
		thePrimitive.initialize(vars, qwery, accum);
		
		return thePrimitive;
	}
	
	public static List list() {
		return new List();
	}
	
	public static TypedObject object(Attribute type){
		return new TypedObject(type);
	}

	public static TypedObject object(){
		return new TypedObject(_ROOT_TYPE_);
	}
	
	public static Pair pair(String name, Attribute value){
		return new Pair(name, value);
	}

	public static Pair pair(String name, Operator operator, Attribute value){
		return new Pair(name, operator, value);
	}
	
	public static Property property(Variable variable, TypedObject object){
		return new Property(variable, object);
	}
	
	public static Negation negation(GenericFact negated) {
		return new Negation(negated);
	}
	
	public static Length length(Attribute list, Attribute length) {
		return new Length(list, length);
	}
}