package edu.vub.crime.grammar;

import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Set;

import edu.vub.crime.grammar.attributes.Attribute;


/** <pre>
 * Contains Expressions,
 * There are specialized constructors to indicate the kind of expressions the list may contain,
 * when inserting a expression to the list a check is done.
 * </pre> */
public class ExpressionList extends Expression {

	private final LinkedList	expressions_ = new LinkedList();
	private final HashSet		itsVariables = new HashSet();
	
	public Iterator iterator() {
		return expressions_.iterator();
	}
	
	public void add(Expression expression) {
		if(expression == null) return;
		
		if (expression instanceof ExpressionList) {
			ExpressionList list = (ExpressionList) expression;
			for (Iterator exprIter = list.expressions_.iterator(); exprIter.hasNext();) {
				Expression expr = (Expression) exprIter.next();
				this.add( expr );
			}
		} else {
			itsVariables.addAll(expression.variables());
			expressions_.add(expression);
		}
	}

	public void addProperty(Property property){
		if(property == null) return;
	
		// This is incorrect, a property should only be added to the most relevant expression.
		// TODO is the most recently added always the best choice?
		Expression expression = (Expression)expressions_.getLast();
		expression.addProperty(property);
		
		
//		property.validate(itsVariables);
//		itsVariables.addAll(property.variables());
//		for(Iterator iter=expressions_.iterator(); iter.hasNext();){
//			Expression expression=(Expression)iter.next();
//			if (expression instanceof ExpressionList) {
//				ExpressionList list = (ExpressionList) expression;
//				list.addProperty(property);
//			}
//			else
//				expression.addProperty(property);
//		}	
	}
	
	public void addAttribute(Attribute attr) {
		throw new RuntimeException("ExpressionList#addAttribute(Attribute) : Should never be called as ExpressionLists are never used as primitive patterns inside the parser.");
	}

	public int size() {	
		return expressions_.size(); 
	}
	
	public Expression getExpression(int index) {
		return (Expression)expressions_.get(index);
	}
	
	public void clear() {
		expressions_.clear();
	}
	
	public void remove(Expression anExpression) {
		expressions_.remove(anExpression);
	}
	
	public void addAll(ExpressionList theOtherExpressions) {
		for (Iterator expressionIt = theOtherExpressions.iterator(); expressionIt.hasNext();) {
			Expression currentExpression = (Expression) expressionIt.next();
			
			add(currentExpression);
		}
	}
	
	public String toString() {
		return expressions_.toString();
	}
	
	public Set variables() {
		return itsVariables;
	}
	
	/**
	 * Inform the visitor of this node's concrete type, by calling its visit method with the 
	 * correct overloaded signature.
	 */
	public void accept(GrammarVisitor visitor) {
		visitor.visit(this);
	}
}