package edu.vub.crime.grammar;


/** <pre>
 * A negation is a special kind of Expression.
 * The rule used for explanation is given below.
 * 
 *         differentTaste(?person, ?genre) :-
 *              isPresent(?person, LivingRoom),
 *              prefers(?person, ?genre),
 *              not prefers(Alice, ?genre).
 *    
 * This rule determines the persons who are in the living room and have a different
 * taste than Alice.
 * 
 * The last condition in the right-hand-side of that rule is a Negation.
 * It is a fact preceeded by the symbol "not".
 * </pre> */
public class Negation extends Expression {
	
	/** <pre>
	 * fact_ : the fact that is negated 
	 * </pre> */
	private GenericFact itsFact;
	
	/** <pre> Do NOT add the variables of the fact to the Negation expression!!
	 * 
	 *  Reason:
	 *  --------------     ----------------------
	 *  | some filter |    | filter Negated Fact |
	 *  --------------     ----------------------
	 *      \                  /
	 *       \                /
	 *        ----------------------------
	 *        |Join with negated combiner|
	 *        ----------------------------
	 *                  |
	 *                  |
	 *         <Only facts from somefilter>
	 *         			|
	 *         		- ----------------
	 *         		| rest of network|
	 *               ----------------
	 *            
	 *  The negated Fact will not bring in new variables because they have to be bound anyway.
	 * </pre> */	
	public Negation(GenericFact theNegatedFact) {
		itsFact = theNegatedFact;
	}
	
	/**
	 * Inform the visitor of this node's concrete type, by calling its visit method with the 
	 * correct overloaded signature.
	 */
	public void accept(GrammarVisitor visitor) {
		visitor.visit(this);
	}
	
	public GenericFact getNegatedFact() {
		return itsFact;
	}
}