package edu.vub.crime.grammar;

import java.util.Set;

import edu.vub.crime.grammar.attributes.Variable;
import edu.vub.crime.grammar.attributes.datastructures.TypedObject;

/**
 * Property class, enable to use the syntax
 * 	:GetPerson(?p):-
 * 		person(?p),
 * 		?p.name.firstname:Bob.
 * 
 *  In this example, the expression is formed as follows:
 *  	+ ?p variable
 *  	+{name:{firstname:Bob}} CrimeObject
 *  
 * 
 */
public class Property extends Primitive {
	private Variable variable;
	private TypedObject object;

	public Property(Variable variable, TypedObject object){
		this.variable = variable;
		this.object = object;
		// TODO object.updateIndex();
		// addAttribute(this.object);
	}
	
	public void validate(Set variables){
		if(!(variables.contains(variable)))
			throw new RuntimeException("Unbound variable "+ variable); 
	}

	public Variable getVariable() {
		return variable;
	}

	public void setVariable(Variable variable) {
		this.variable = variable;
	}

	public TypedObject getObject() {
		return object;
	}
	
	public void accept(GrammarVisitor visitor) {
		visitor.visit(this);
	}
	
	
}
