package edu.vub.crime.grammar;

import java.util.Iterator;

import edu.vub.crime.CrimeEngine;
import edu.vub.crime.rete.data.Token;

/** <pre>
 * A rule consits of one or more conditions (prerequisites),
 * and one or more conseauences.
 * The rule used for explanation is given below.
 * 
 *         differentTaste(?person, ?genre) :-
 *              isPresent(?person, LivingRoom),
 *              prefers(?person, ?genre),
 *              not prefers(Alice, ?genre).      
 *              
 * This rule determines the persons who are in the living room and have a different
 * taste than Alice.
 *              
 * This rule had three conditions, namely
 *         - isPresent(?person, LivingRoom)
 *         - prefers(?person, ?genre)
 *         - not prefers(Alice, ?genre)
 * The rule has only one consequence, namely differentTaste(?person, ?genre).
 * </pre> */
public class Rule extends Expression {
	
	/** <pre>
	 * consequence_ : 
	 * prerequisite_ : 
	 * </pre> */
	private final ExpressionList consequence_;
	private final ExpressionList prerequisite_;

	/** <pre>
	 * Constructor.
	 * </pre> */
	public Rule(ExpressionList consequences, ExpressionList prerequisite) {		
		this.consequence_ = consequences;
		this.prerequisite_ = prerequisite;
	}
	
	/** <pre>
	 * Accessor for datamember consequence_.
	 * </pre> */
	public ExpressionList getConsequence() {
		return consequence_;
	}

	/** <pre>
	 * Accessor for the prerequisites.
	 * </pre> */
	public ExpressionList getPrerequisite() {
		return prerequisite_;
	}

	/** <pre>
	 * Executing, given a certain token 
	 * executing all the events of the rule.
	 * </pre> */
	public void execute(CrimeEngine engine, Token token) {
		for(Iterator it=consequence_.iterator(); it.hasNext(); ) {		
			Event event = (Event) it.next();
			event.produce(token);
		}
	}
	
	/**
	 * Inform the visitor of this node's concrete type, by calling its visit method with the 
	 * correct overloaded signature.
	 */
	public void accept(GrammarVisitor visitor) {
		visitor.visit(this);
	}
	
	public String toString() {
		return consequence_.toString() + " :- " + prerequisite_.toString();
	}
}