package edu.vub.crime.grammar.attributes;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import edu.vub.crime.grammar.attributes.datastructures.List;

/** 
 * The CompoundAttribute class is the common root for all Crime datastructures.
 * Since structures can be composites, it is no longer sufficient to describe 
 * attributes by a single index (i.e. their position in the fact) but instead
 * require a list of indices which describe the index at every level. Moreover,
 * since the introduction of structures allows indexing based on a name, this
 * index is no longer necessarily a number.
 * 
 * @author mhernand
 * @author smostinc   
 */
public abstract class CompoundAttribute extends Attribute {
	protected Vector	itsAttributes = new Vector();
	protected Set		itsVariables  = new HashSet();

	protected boolean	isNil;
	
	public Iterator iterator() {
		return itsAttributes.iterator();
	}
	
	public boolean isEmpty() {
		return isNil;
	}
	
	/**
	 * Dispatches on the actual type of the attribute to perform the correct operations.
	 */
	public void addAttribute(Attribute theAttribute) {
		if (theAttribute instanceof Variable) {
			impl_addAttribute((Variable) theAttribute);
		} else if (theAttribute instanceof CompoundAttribute) {
			impl_addAttribute((CompoundAttribute) theAttribute);
		} else {
			impl_addAttribute(theAttribute);
		}
	}
	
	private void impl_addAttribute(Attribute theAttribute) {
		itsAttributes.add(theAttribute);
	}
	
	private void impl_addAttribute(Variable theVariable) {
		itsVariables.add(theVariable);
		
		// Call the 'super' implementation
		impl_addAttribute((Attribute)theVariable);
	}
	
	private void impl_addAttribute(CompoundAttribute theCompoundAttribute) {
		itsVariables.addAll(theCompoundAttribute.getVariables());
		
		// Call the 'super' implementation
		impl_addAttribute((Attribute)theCompoundAttribute);
	}
	
	public Set getVariables() {
		return itsVariables;
	}
		
	public Vector attributes() {
		return itsAttributes;
	}
	
	public void remove(Attribute attr) {
		itsAttributes.remove(attr);
	}

	public List copyAttributes(List attributes) {
		for(Iterator it=itsAttributes.iterator(); it.hasNext(); ) {
			Attribute attr = (Attribute) it.next();
			attributes.add(attr);
		}
		return attributes;
	}

	public boolean matches(Attribute t) {
		return this.equals(t);
	}

}
