package edu.vub.crime.grammar.attributes;

import edu.vub.crime.grammar.ExpressionList;
import edu.vub.crime.grammar.utils.AttributePicker;
import edu.vub.crime.grammar.utils.UtilsFactory;

/**
 * This abstract class is used to group all attributes which encapsulate a value that cannot
 * be split up further.  Additionally, subclasses guarantee that their value will not change
 * during the course of the program. 
 * <p>
 * Grouping these attributes is beneficial,  since they are usually treated uniformly by all
 * attribute visitor subclasses.  By means of this abstract superclass,  it becomes possible
 * to identify them and writing a single handler.
 * 
 * @author smostinc
 */
public abstract class ConstantAttribute extends Attribute {

	public AttributePicker makeAttributePicker(ExpressionList facts) {
		return UtilsFactory.ctePicker((Attribute)this.clone());
	}
	
	protected void cloneIndexFor(Attribute anAttribute) {
		anAttribute.setIndex(getIndex());
	}
}
