package edu.vub.crime.grammar.attributes.constant;

import edu.vub.crime.grammar.ExpressionFactory;
import edu.vub.crime.grammar.attributes.Attribute;
import edu.vub.crime.grammar.attributes.AttributeVisitor;
import edu.vub.crime.grammar.attributes.ConstantAttribute;

/** 
 * Numeric attribute which holds a double precision floating point number.
 * It is possible to compare a fraction with an ordinary number.
 */
public class Fraction extends ConstantAttribute {

	private final double itsValue;
	
	public Fraction(double aFraction)				{ itsValue = aFraction; }
	public double getValue()						{ return itsValue; }
	public String toString() 						{ return Double.toString(itsValue); }
	public void accept(AttributeVisitor theVisitor) { theVisitor.visit(this); }
	
	public boolean equals(Object anotherObject) {
		if(anotherObject instanceof Number)
			return itsValue == ((Number)anotherObject).getValue();
		
		if(anotherObject instanceof Fraction)
			return itsValue == ((Fraction)anotherObject).getValue();
		
		return false;
	}

	public Object clone() {		
		 Attribute theClone = ExpressionFactory.attribute(itsValue);
		 cloneIndexFor(theClone);
		 return theClone;
	}
}