package edu.vub.crime.grammar.attributes.datastructures;

import edu.vub.crime.grammar.attributes.Variable;

/**
 * The RestVariable class is a marker class which is used to differentiate between ordinary
 * variables and variables which appear in a tail position in a list expression, such as the
 * <tt>?rest</tt> variable in the following rule: 
 * <pre>
 * :log(["Business Contact:", ?firstName, ?lastName]) :- 
 *   addressBook([?firstName, ?lastName | ?rest]).
 * </pre>
 * Note that <tt>?lastName</tt> is not a RestVariable, although it is the last variable in the
 * list of the resulting event. The difference is the <tt>?rest</tt> is preceded by a pipe (<tt>
 * |</tt>), which implies that the variable may capture any amount of remaining elements in the
 * list.
 * <p>
 * Whether a variable is a rest variable can also be stored using a boolean in the Variable itself,
 * yet this is suboptimal since it requires a boolean to be present in all variables, most of which
 * don't need it. A dedicated type allows for similarly simple tests and has no overhead for normal
 * variables.
 * 
 * @author smostinc
 */
public class RestVariable extends Variable {

	public RestVariable(Variable theVariable) {
		super(theVariable.getName());
	}
	
	public Object clone() {
		return new RestVariable((Variable)super.clone());
	}
}
