package edu.vub.crime.grammar.utils;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import edu.vub.crime.exceptions.IncorrectAttributeIndexException;
import edu.vub.crime.grammar.ExpressionFactory;
import edu.vub.crime.grammar.attributes.datastructures.TypedObject;
import edu.vub.crime.rete.data.Token;

public class ActivationBuilder {

	private final String 	itsActivationType;
	private final HashMap	itsAttributePickers;
	
	public ActivationBuilder(String theActivationType) {
		itsActivationType	= theActivationType;
		itsAttributePickers	= new HashMap();
	}
	
	/**
	 * @param theAttributeName the name under which the attribute will be passed in the map
	 * @param thePicker a picker which can produce an attribute value given a token
	 */
	public void addPicker(String theAttributeName, AttributePicker thePicker) {
		itsAttributePickers.put(theAttributeName, thePicker);
	}

    public String getType() { return itsActivationType; }

    public TypedObject buildActivation(Token token) {
    	try {
    		TypedObject theActivation = ExpressionFactory.object(ExpressionFactory.type(itsActivationType));
			
			for (Iterator entryIt = itsAttributePickers.entrySet().iterator(); entryIt.hasNext();) {
				Map.Entry currentEntry 		= (Map.Entry) entryIt.next();
				
				String theAttributeName		= (String) currentEntry.getKey();
				AttributePicker thePicker	= (AttributePicker) currentEntry.getValue();
				
				theActivation.addField(ExpressionFactory.pair(theAttributeName, thePicker.pickAttribute(token)));
			}
			
			return theActivation;
		} catch (IncorrectAttributeIndexException e) {
			throw new RuntimeException("Error building a fact: " + e.getMessage(), e);
		}
    }
}