package edu.vub.crime.grammar.utils;

import edu.vub.crime.exceptions.IncorrectAttributeIndexException;
import edu.vub.crime.grammar.attributes.Attribute;
import edu.vub.crime.rete.data.Token;

/** 
 * The AttributePicker class is the abstract superclass for all attribute pickers.
 * An attribute picker is used to extract variable values from a Token in order to
 * pass them to construct an event. The production node of the following rule needs 
 * to extract the value of the <tt>?other</tt> variable from any token it is passed.
 * <pre>
 * :showIcon(?other) :-
 *   public -> colocated(me, ?other).
 * </pre>
 * <p>
 * Therefore the event needs an attribute picker which selects the second attribute
 * of the first fact, for every token that is passed to it. Since events can also
 * contains constants or datastructures, a class hierarchy is created to allow many
 * different pickers.
 * 
 * @author cfscholl
 * @author ephilips
 */
public abstract class AttributePicker {
	 
	/** 
     * Abstract method: all attribute pickers provide a mechanism to 
     * produce an attribute from a token.
	 * @throws IncorrectAttributeIndexException 
     */
    public abstract Attribute pickAttribute(Token token) throws IncorrectAttributeIndexException;	
}
