package edu.vub.crime.grammar.utils;

import edu.vub.crime.grammar.attributes.Attribute;
import edu.vub.crime.rete.data.Token;


/** 
 * The ConstantPicker class is an attribute picker which always returns fresh clones of 
 * a preconfigured attribute. This is required to allow for rules where the event on the 
 * lefthand side of a rule contains constants such as:
 * <pre>
 * :log("Detected a colocated user") :-
 *   public -> colocated(me, ?user).
 * </pre>
 * 
 * @author cfscholl
 * @author ephilips
 */
public class ConstantPicker extends AttributePicker{
	
    private Attribute attribute_;

    /** 
     * Constructor taking the attribute value to be returned when the picker is activated
     */
    public ConstantPicker(Attribute attribute) {
        this.attribute_ = attribute;
    }

    /** 
     * Returns a clone of the preconfigured attribute regardless of the passed token.
     */
    public Attribute pickAttribute(Token token) {
        return (Attribute) this.attribute_.clone();
    }
}