package edu.vub.crime.grammar.utils;

import edu.vub.crime.exceptions.IncorrectAttributeIndexException;
import edu.vub.crime.grammar.ExpressionList;
import edu.vub.crime.grammar.attributes.Attribute;
import edu.vub.crime.grammar.attributes.datastructures.Pair;
import edu.vub.crime.rete.data.Token;

/**
 * The PairPicker attribute picker is used to construct a single pair when a literal
 * object is passed on to an event. Pair pickers are always parts of an aggregated 
 * {@link ObjectPicker}. Consider the following rule:
 * <pre>
 * :grade({ student: ?s; course: "Structure of Computer Programs"}) :-
 *   public -> enrolled(?s, ?program),
 *   requires(?program, "Structure of Computer Programs").
 * </pre>
 * The Object picker for the <tt>:grade</tt> event will contain two pair pickers, one
 * to construct the student pair and one to construct the course pair.
 * 
 * @author mhernand
 * @author smostinc
 */
public class PairPicker extends AttributePicker {
	
	private final AttributePicker itsConstituentPicker;
	private final String theNameOfTheConstructedPair;
	
	/**
	 * Constructor which implicitly creates a nested attribute picker to retrieve
	 * the pair's associated value.
	 * 
	 * @param theFacts in the order in which they will be present in the tokens that will be passed to the picker
	 * @param thePair for which a picker is being created.
	 */
	public PairPicker(ExpressionList theFacts, Pair thePair){
		theNameOfTheConstructedPair  = thePair.getName();

		Attribute thePairItsValueAttribute = thePair.getValue();
		itsConstituentPicker = thePairItsValueAttribute.makeAttributePicker(theFacts);
	}

	/**
	 * Returns a new pair with the picked attribute value.
	 */
	public Attribute pickAttribute(Token token) throws IncorrectAttributeIndexException {
		return new Pair(theNameOfTheConstructedPair, itsConstituentPicker.pickAttribute(token));
	}

}
