package edu.vub.crime.memory;

import java.util.Iterator;
import java.util.Vector;

import edu.vub.crime.grammar.GenericFact;

/** <pre>
 * A FactSpace contains all facts of a certain space 
 * (for example private, public, jukebox, ...).
 * </pre> */
public class FactSpace {

	/** <pre>
	 * facts_ : contains all the facts
	 * space_ : the name of the fact space
	 * </pre> */
	private Vector facts_;
	private final String space_;
	
	/** <pre>
	 * Constructor.
	 * </pre> */
	public FactSpace(String space) {
		space_ = space;
		facts_ = new Vector();
	}
	
	/** <pre>
	 * Add a fact.
	 * </pre> */
	public void addFact(GenericFact fact) {
		facts_.add(fact);
	}
	
	/** <pre>
	 * Remove a given fact.
	 * </pre> */
	public void removeFact(GenericFact fact) {
		facts_.remove(fact);
	}
	
	/** <pre>
	 * Check if a given fact was already added or not.
	 * Returns a boolean : true when the fact was already inserted, false otherwise.
	 * </pre> */
	public boolean check(GenericFact fact) {
		for(Iterator it=facts_.iterator(); it.hasNext(); ) {
			GenericFact memFact = (GenericFact) it.next();
			if(memFact.equals(fact)) {
				return true;
			}
		}
		return false;
	}
	
	/** <pre>
	 * Return the iterator of the facts_ datamember.
	 * </pre> */
	public Iterator iterator() {
		return facts_.iterator();
	}
	
	/** <pre>
	 * Return all facts.
	 * </pre> */
	public Vector facts() {
		return facts_;
	}
	
	/** <pre>
	 * Return the number of facts.
	 * </pre> */
	public int size() {
		return facts_.size();
	}
	
	/** <pre>
	 * Return the name of the space.
	 * </pre> */
	public String space() {
		return space_;
	}
}
