package edu.vub.crime.memory;

import edu.vub.crime.grammar.GenericFact;

/**
 * The Memory interface describes the expected methods for any memory of a CrimeEngine. Normally the 
 * Memory only serves as a repository of those facts that are currently known, yet in a networked 
 * setting, this memory is also used to group facts according to their publisher, such that when the
 * latter disappears all the facts that need to be removed are known.
 * 
 * @author smostinc
 */
public interface Memory {

	/**
	 * Commit a new fact to memory, in the proper fact space. If this fact space was
	 * previously unknown, it is added to the memory.
	 */
	public abstract void insert(GenericFact fact);

	/** 
	 * Forget about the existence of a fact. 
	 */
	public abstract void remove(GenericFact fact);

}