package edu.vub.crime.network;

import java.util.Properties;

import junit.framework.TestCase;
import edu.vub.crime.CrimeFactory;

/**
 * Subclasses of this class are equipped with two independent crime engines which are manually connected to
 * one another.  In other words, public facts asserted in either engine are asserted in the other engine as
 * well. This provides a controlled setup to test the network behavior of a Crime Engine, without requiring
 * actual distributed setup.
 * 
 * @author smostinc
 */
public abstract class DualEngineTestCase extends TestCase {

	/**
	 * This protected constructor ensures that the classes that are tested by the standard suite
	 * of unit tests use the default Crime engine without any network interface.
	 */
	protected DualEngineTestCase() {
		Properties theClassesUnderTest = CrimeFactory.factoryClasses;
		
		theClassesUnderTest.setProperty("Engine",		"edu.vub.crime.network.ConnectedCrimeEngine");
		theClassesUnderTest.setProperty("Memory",		"edu.vub.crime.memory.NetworkMemory");
		theClassesUnderTest.setProperty("Network",		"edu.vub.crime.network.NetworkLayer");
		theClassesUnderTest.setProperty("Agenda",		"edu.vub.crime.rete.agenda.FIFOAgenda");
		theClassesUnderTest.setProperty("Scheduler",	"edu.vub.crime.rete.agenda.InlineScheduler");

	}
}
