package edu.vub.crime.network;

import edu.vub.crime.CrimeEngine;
import edu.vub.crime.rete.agenda.Activation;

/**
 * The NetworkLayer class specifies the interface through which CRIME engines may communicate with the outside
 * world and provides a default implementation for these methods which does nothing. To use a different network
 * interface, one must create a subclass of this class and ensure that the CRIME factory will load that custom
 * class.
 * 
 * @author smostinc
 */
public class NetworkLayer {

	protected CrimeEngine itsEngine;
	
	public NetworkLayer(CrimeEngine theEngine) {
		itsEngine = theEngine;
	}
	
	/**
	 * This method is used to publish an activation (i.e. a combination of assertions and/or retractions of facts) 
	 * which was derived by this CRIME engine (as opposed to being received from another engine). The network layer
	 * is responsible to advertise this activation to all connected CRIME engines.
	 * <p>
	 * By default CRIME engines use this class to communicate with the network which assumes that there are no
	 * connected engines and that the activation can therefore be ignored.
	 * 
	 * @param theTokens the activation which should be published to all connected engines.
	 */
	public void publishActivation(Activation theTokens) {}

}
