/**
 * 
 */
package edu.vub.crime.network.jgroups;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Vector;

import org.apache.log4j.PropertyConfigurator;
import org.jgroups.Address;
import org.jgroups.Channel;
import org.jgroups.JChannel;
import org.jgroups.blocks.GroupRequest;
import org.jgroups.blocks.MethodCall;
import org.jgroups.blocks.RpcDispatcher;

import edu.vub.crime.CrimeEngine;
import edu.vub.crime.grammar.GenericFact;
import edu.vub.crime.memory.NetworkMemory;
import edu.vub.crime.network.NetworkLayer;
import edu.vub.crime.rete.agenda.Activation;
import edu.vub.crime.rete.agenda.ExternalActivation;
import edu.vub.crime.rete.data.TokenTag;
import edu.vub.crime.util.Logging;

/**
 * @author smostinc
 *
 */
public class JGroupsNetworkLayer extends NetworkLayer {

	static {
		PropertyConfigurator.configure("log4j.properties");
		// BasicConfigurator.configure();
	}
	
	private final NetworkMemory theMemory;

	JChannel 		itsChannel;
	Address  		itsLocalAddress;
	RpcDispatcher	itsDispatcher;
	
	public JGroupsNetworkLayer(CrimeEngine theEngine) throws Exception {
		super(theEngine);
		
		Logging.JGroups_LOG.info("initializing JGroups Channel");
		
		theMemory = (NetworkMemory)theEngine.getMemory();

		String network_name = System.getProperty("crime.network");

		itsChannel = new JChannel();
		itsChannel.setOpt(Channel.LOCAL, Boolean.FALSE);

		itsDispatcher = new RpcDispatcher(itsChannel, null, new MessageReceiver(itsEngine, this), this);
		itsChannel.connect(network_name);
	}
	
	public Serializable getLocalAddress() {
		if(itsLocalAddress == null)
			itsLocalAddress = itsChannel.getLocalAddress();
		
		return itsLocalAddress;
	}
	
	public void disconnectedSource(Serializable source) {
		if(source.equals(getLocalAddress())) return;
		
		Logging.JGroups_LOG.info("A possible source of context information disconnected : " + source);
		
		theMemory.remove(source.toString(), itsEngine);
	}
	
	
	public void connectedSource(Serializable source) {
		if(source.equals(getLocalAddress())) return;
		
		Logging.JGroups_LOG.info("A possible source of context information was discovered : " + source);
		
		// TODO Send all public facts published by myself
		Vector myFacts = theMemory.facts("_THIS_");
		
		for (Iterator factIt = myFacts.iterator(); factIt.hasNext();) {
			GenericFact currentFact = (GenericFact) factIt.next();
			
			currentFact.setPublisher(getLocalAddress().toString());
			
			try {
				itsDispatcher.callRemoteMethod(
						(Address)source, 
						new MethodCall("assert", new Object[] {currentFact}, new Class[] { GenericFact.class }), 
						GroupRequest.GET_NONE,
						10000);
			} catch (Throwable e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public void assert(GenericFact theFact) {
		itsEngine.insertActivation(new ExternalActivation(TokenTag.add, theFact, theFact.getPublisher()));
	}
	
	public void acceptActivation(Activation theActivation) {
//		System.out.println("Accepting an activation" + theActivation);
//		
//		if (theActivation instanceof ExternalActivation) {
//			ExternalActivation theExternalActivation = (ExternalActivation) theActivation;
//			
//			if(theExternalActivation.getPublisher() == this.toString()) return;
//		}
//		
		itsEngine.insertActivation(theActivation);
	}
	
	public void publishActivation(Activation theTokens) {
		itsDispatcher.callRemoteMethods(
				null, // broadcast to all connected engines
				new MethodCall("acceptActivation", new Object[] { theTokens }, new Class[] { Activation.class }), 
				GroupRequest.GET_NONE, 
				1000);
	}
	
	public String toString() {
		return itsLocalAddress.toString();
	}
}
