package edu.vub.crime.network.jgroups;

import java.util.Iterator;
import java.util.Vector;

import org.jgroups.Address;
import org.jgroups.Message;
import org.jgroups.ReceiverAdapter;
import org.jgroups.View;

import edu.vub.crime.CrimeEngine;
import edu.vub.crime.rete.agenda.Activation;

public class MessageReceiver extends ReceiverAdapter {
	
	Vector itsGroupMembers;
	CrimeEngine itsEngine;
	JGroupsNetworkLayer itsNetworkLayer;
	
	public MessageReceiver(CrimeEngine itsEngine,
			JGroupsNetworkLayer itsNetworkLayer) {
		super();
		this.itsGroupMembers = new Vector();
		this.itsEngine = itsEngine;
		this.itsNetworkLayer = itsNetworkLayer;
	}

	public void receive(Message theMessage) {
		Activation theActivation = (Activation) theMessage.getObject();
		
		itsEngine.insertActivation(theActivation);
	};
	
	public void viewAccepted(View theUpdatedView) {
        if(theUpdatedView == null) return;

        Vector theJoinedMembers = new Vector();
        Vector theLeftMembers = new Vector();
        
        Vector theNewViewContents = theUpdatedView.getMembers();
        Object theNewViewMember;

        synchronized(itsGroupMembers) {
            // get new members
            for(int i=0; i < theNewViewContents.size(); i++) {
                theNewViewMember=theNewViewContents.elementAt(i);
                if(!itsGroupMembers.contains(theNewViewMember))
                	theJoinedMembers.addElement(theNewViewMember);
            }

            // get members that left
            for(int i=0; i < itsGroupMembers.size(); i++) {
                theNewViewMember=itsGroupMembers.elementAt(i);
                if(!theNewViewContents.contains(theNewViewMember))
                	theLeftMembers.addElement(theNewViewMember);
            }

            // adjust our own membership
            itsGroupMembers.removeAllElements();
            itsGroupMembers.addAll(theNewViewContents);
        }

		for (Iterator disconnectedIt = theLeftMembers.iterator(); disconnectedIt.hasNext();) {
			Address currentDisconnectedPeer = (Address) disconnectedIt.next();
			
			itsNetworkLayer.disconnectedSource(currentDisconnectedPeer);
		}
		
		for (Iterator disconnectedIt = theJoinedMembers.iterator(); disconnectedIt.hasNext();) {
			Address currentDisconnectedPeer = (Address) disconnectedIt.next();
			
			itsNetworkLayer.connectedSource(currentDisconnectedPeer);
		}
	}
}
