/**
 * 
 */
package edu.vub.crime.network.m2mi;

import edu.rit.m2mi.Eoid;

/**
 * Objects can export themselves as implementors of this interface so that they can be regularly 
 * notified of all CrimeEngines in the environment. When several heartbeat messages are missed, 
 * the CrimeEngine is reported to be disconnected, and the facts it published will be retracted.
 * 
 * @author smostinc
 */
public interface HeartbeatRef {

	/**
	 * The time in milliseconds before a Crime Engine is presumed to be disconnected (30 sec).
	 */
	static final int LEASE_TIME = 30000;
	
	/**
	 * This method is invoked to signal that the given engine is still available and that its
	 * facts should not yet be retracted. This message is normally received between (0.2 * lease 
	 * time) and (0.4 lease time), which implies that at least two messages should arrive before
	 * the lease expires. This implies that one message can be safely missed.
	 * 
	 * @param anEngine the unique id of the engine which sent the message.
	 */
	public void beat(Eoid anEngine);
}
