package edu.vub.crime.network.m2mi.sync;

public class SlidingWindow {

	private int itsTotal 		= 0;
	private int itsOffset 		= 0;
	private int itsRetracted	= 0;
	
	private Object[] itsData 	= new Object[8];
	
	public SlidingWindow() {}
	
	public SlidingWindow(int retracted, int total) {
		itsRetracted 	= retracted;
		itsTotal 		= total;
	}
	
	private static class Retracted {
		public String toString() {
			return "Retracted";
		}
	};	
	
	public boolean assert(Object fact, int total) {
		int theIndex = normalize(total - itsRetracted + itsOffset);
		
		Object currentValue = itsData[theIndex];
		
		if(currentValue != null && !(currentValue instanceof Retracted)) {
			// the array has been fully wrapped around, increase the available storage
			this.doubleCapacity();
			
			// retry the assertion with the new storage in place
			return this.assert(fact, total);
		}
		
		itsData[theIndex] = fact;
		
		if(total >= itsTotal)
			itsTotal = total+1;
		
		return true;
	}
	
	public Object getFact(int index) {
		if(index < itsRetracted || index > itsTotal)
			return null;
		
		int theIndex = normalize(index - itsRetracted + itsOffset);
		
		return itsData[theIndex];
	}
	
	public boolean retract(Object fact, int retract) {
		int theIndex = indexOf(fact);
		
		if(theIndex == -1)
			// if the retracted fact cannot be found, report to the client code
			return false;
		
		int theRetractedIndex = normalize(retract - itsRetracted + itsOffset);
		Object theSwappedFact = itsData[theRetractedIndex];
		
		itsData[theIndex] = theSwappedFact;
		itsData[theRetractedIndex] = new Retracted();
		
		updateRetractionCounter();
		
		return true;
	}
	
	public String toString() {
		StringBuffer theBuffer = new StringBuffer();
		
		theBuffer.append('[');
		
		String separatorToken = "";
		
		int activeElements = itsTotal - itsRetracted;
		
		for(int elementCount = 0; elementCount < activeElements ; elementCount++) {
			int currentIndex = normalize(elementCount + itsOffset);
			
			theBuffer.append(separatorToken);
			theBuffer.append(itsData[currentIndex]);
			
			separatorToken = ",";
		}
		
		theBuffer.append(']');
		
		return theBuffer.toString();
	}
	
	public void clear() {
		itsTotal 	 = 0;
		itsOffset 	 = 0;
		itsRetracted = 0;
		itsData 	 = new Object[8];
	}
	
	private void updateRetractionCounter() {
		Object lastRetracted = itsData[normalize(itsRetracted)];
		
		while (lastRetracted != null && lastRetracted instanceof Retracted) {
			itsOffset++;
			itsRetracted++;
			
			lastRetracted = itsData[normalize(itsRetracted)];
		}
	}

	private void doubleCapacity() {
		Object[] theNewData = new Object[itsData.length * 2];
		
		int copyIndex = 0;
		int originIndex = itsOffset;
		
		int activeElements = itsTotal - itsRetracted;
		
		for(; copyIndex < activeElements ;) {
			theNewData[copyIndex++] = itsData[normalize(originIndex++)];
		}
		
		itsData = theNewData;
		itsOffset = 0;
	}
	
	private int normalize(int index) {
		return index % itsData.length;
	}
	
	private int indexOf(Object fact) {
		int activeElements = itsTotal - itsRetracted;
		
		for(int elementCount = 0; elementCount < activeElements ; elementCount++) {
			int currentIndex = normalize(elementCount + itsOffset);

			if(fact.equals(itsData[currentIndex]))
				return currentIndex;
		}
		
		
		return -1;
	}

	int findMissingAssertion(int start, int stop) {
		int theResult = start;
		
		for(; theResult < stop; theResult++) {
			int currentIndex = normalize(theResult - itsRetracted + itsOffset);
			
			Object currentValue = itsData[currentIndex];
			
			if(currentValue == null  || (currentValue instanceof Retracted))
				break;
		}
		
		return theResult;
	}

	int findMissingRetraction(int start, int stop) {
		int theResult = start;
		
		for(; theResult < stop; theResult++) {
			int currentIndex = normalize(theResult - itsRetracted + itsOffset);
			
			Object currentValue = itsData[currentIndex];
			
			if(! (currentValue instanceof Retracted))
				break;
		}
		
		return theResult;
	}
}
