package edu.vub.crime.network.m2mi.sync;

import edu.rit.m2mi.Eoid;

public abstract class SynchronisationState {

	/**
	 * Identifies the device for which we are keeping track of the asserted and retracted facts.
	 */
	protected Eoid itsDevice;
	
	/**
	 * The total amount of facts that have been asserted by a particular device. This
	 * count is monotonically increasing and is larger than or equal to its retraction
	 * count.
	 */
	protected int itsAssertionCount;
	
	/**
	 * The total amount of facts that have been retracted by a particular device. This
	 * count is monotonically increasing and is less than or equal to the assertion 
	 * count.
	 */
	protected int itsRetractionCount;
	
	/**
	 * The SlidingWindow keeps a continuous range of facts which are currently true. This
	 * is used to synchronize different fact spaces in a peer to peer fashion. 
	 */
	protected SlidingWindow itsSlidingWindow;
	
	protected SynchronisationState(SlidingWindow facts, Eoid device, int retract, int total) {
		itsSlidingWindow = facts;
		itsDevice = device;
		
		itsRetractionCount = retract;
		itsAssertionCount = total;
	}
	
	public abstract SynchronisationState beat(int retracted, int total);
	
	public abstract SynchronisationState assert(String fact, int retracted, int total);
	
	public abstract SynchronisationState retract(String fact, int retracted, int total);
}
