package edu.vub.crime.network.m2mi.sync;

import junit.framework.TestCase;

public class TestSlidingWindow extends TestCase {

	private SlidingWindow itsSlidingWindow;
	
	protected void setUp() throws Exception {
		itsSlidingWindow = new SlidingWindow();
	}
	
	public void testAssertion() {
		itsSlidingWindow.assert("f0", 0);
		
		assertEquals("[f0]", itsSlidingWindow.toString());
		
		itsSlidingWindow.assert("f2", 2);

		assertEquals("[f0,null,f2]", itsSlidingWindow.toString());

		itsSlidingWindow.assert("f1", 1);

		assertEquals("[f0,f1,f2]", itsSlidingWindow.toString());
	}
	
	public void testSimpleRetraction() {
		itsSlidingWindow.assert("f0", 0);
		itsSlidingWindow.assert("f2", 2);
		itsSlidingWindow.assert("f1", 1);
		itsSlidingWindow.assert("f3", 3);
		
		itsSlidingWindow.retract("f1", 0);
		
		assertEquals("[f0,f2,f3]", itsSlidingWindow.toString());
		
		itsSlidingWindow.retract("f2", 1);
		
		assertEquals("[f0,f3]", itsSlidingWindow.toString());

		itsSlidingWindow.assert("f4", 4);
		
		assertEquals("[f0,f3,f4]", itsSlidingWindow.toString());
	}
	
	public void testAdvancedRetraction() {
		itsSlidingWindow.assert("f0", 0);
		itsSlidingWindow.assert("f2", 2);
		itsSlidingWindow.assert("f1", 1);
		itsSlidingWindow.assert("f3", 3);
		
		itsSlidingWindow.retract("f2", 1);
		
		assertEquals("[f0,Retracted,f1,f3]", itsSlidingWindow.toString());
		
		itsSlidingWindow.retract("f1", 0);
		
		assertEquals("[f0,f3]", itsSlidingWindow.toString());

		itsSlidingWindow.assert("f4", 4);
		
		assertEquals("[f0,f3,f4]", itsSlidingWindow.toString());
	}
	
	public void testGetFact() {
		itsSlidingWindow.assert("f0", 0);
		itsSlidingWindow.assert("f2", 2);
		itsSlidingWindow.assert("f1", 1);
		itsSlidingWindow.assert("f3", 3);
		
		assertEquals("f2", itsSlidingWindow.getFact(2));
		
		itsSlidingWindow.retract("f2", 1);
		
		assertEquals("f1", itsSlidingWindow.getFact(2));
		
		itsSlidingWindow.retract("f1", 0);
		
		assertEquals("f0", itsSlidingWindow.getFact(2));
		
		itsSlidingWindow.retract("f3", 2);
		
		assertNull(itsSlidingWindow.getFact(2));
		
	}

	public void testMissingAssertionIndex() {
		itsSlidingWindow.assert("f3", 3);
		
		assertEquals(0, itsSlidingWindow.findMissingAssertion(0, 3));

		itsSlidingWindow.assert("f1", 1);
		
		assertEquals(0, itsSlidingWindow.findMissingAssertion(0, 3));

		itsSlidingWindow.assert("f0", 0);
		
		assertEquals(2, itsSlidingWindow.findMissingAssertion(0, 3));

		itsSlidingWindow.assert("f2", 2);
		
		assertEquals(3, itsSlidingWindow.findMissingAssertion(0, 3));
	
	}
}
