package edu.vub.crime.network.m2mi.sync;

import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

public class TestSynchronisationState extends TestCase {

	private static Map ASSERTION_TRANSITION;
	private static Map ASSERTION_GAP_TRANSITION;

	private static Map RETRACTION_TRANSITION;
	private static Map RETRACTION_GAP_TRANSITION;

	
	static {
		ASSERTION_TRANSITION = new HashMap();
		
		ASSERTION_TRANSITION.put(NoGaps.class, 				NoGaps.class);
		ASSERTION_TRANSITION.put(AssertionGap.class, 		NoGaps.class);
		ASSERTION_TRANSITION.put(RetractionGap.class, 		RetractionGap.class);
		ASSERTION_TRANSITION.put(DoubleGap.class, 			RetractionGap.class);

		ASSERTION_GAP_TRANSITION = new HashMap();
		
		ASSERTION_GAP_TRANSITION.put(NoGaps.class, 			AssertionGap.class);
		ASSERTION_GAP_TRANSITION.put(AssertionGap.class, 	AssertionGap.class);
		ASSERTION_GAP_TRANSITION.put(RetractionGap.class, 	DoubleGap.class);
		ASSERTION_GAP_TRANSITION.put(DoubleGap.class, 		DoubleGap.class);

		RETRACTION_TRANSITION = new HashMap();
		
		RETRACTION_TRANSITION.put(NoGaps.class, 			NoGaps.class);
		RETRACTION_TRANSITION.put(RetractionGap.class, 		NoGaps.class);
		RETRACTION_TRANSITION.put(AssertionGap.class, 		AssertionGap.class);
		RETRACTION_TRANSITION.put(DoubleGap.class, 			AssertionGap.class);

		RETRACTION_GAP_TRANSITION = new HashMap();
		
		RETRACTION_GAP_TRANSITION.put(NoGaps.class, 		RetractionGap.class);
		RETRACTION_GAP_TRANSITION.put(RetractionGap.class, 	RetractionGap.class);
		RETRACTION_GAP_TRANSITION.put(AssertionGap.class, 	DoubleGap.class);
		RETRACTION_GAP_TRANSITION.put(DoubleGap.class, 		DoubleGap.class);
	}
	
	private SynchronisationState itsSyncState;
	
	private void verifySyncState(SynchronisationState theNewState, boolean shouldBeEqual, Class expectedClass) {
		if(shouldBeEqual) {
			assertSame(itsSyncState, theNewState);
		} else {
			assertNotSame(itsSyncState, theNewState);
		}
		
		assertTrue(expectedClass.isInstance(theNewState));
		
		// When all assertions succeeded, use the new state as the official sync state
		itsSyncState = theNewState;
	}
		
	public void testRetractionGap() {
		itsSyncState = new NoGaps(new SlidingWindow(), null, 0, 0);
	
		assertSame(itsSyncState, itsSyncState.assert("f0", 0, 0));
		assertSame(itsSyncState, itsSyncState.assert("f1", 0, 1));
		assertSame(itsSyncState, itsSyncState.assert("f2", 0, 2));
		assertSame(itsSyncState, itsSyncState.assert("f3", 0, 3));
		assertSame(itsSyncState, itsSyncState.assert("f4", 0, 4));
		assertSame(itsSyncState, itsSyncState.assert("f5", 0, 5));
		
		assertEquals("[f0,f1,f2,f3,f4,f5]", itsSyncState.itsSlidingWindow.toString());
		
		SynchronisationState temp = itsSyncState.beat(1, 5);
		
		assertNotSame(itsSyncState, temp);
		
		assertTrue(temp instanceof RetractionGap);
		
		itsSyncState = temp;
		
		assertSame(itsSyncState, itsSyncState.retract("f2", 1, 5));
		
		assertEquals("[f0,Retracted,f1,f3,f4,f5]", itsSyncState.itsSlidingWindow.toString());
		
		assertSame(itsSyncState, itsSyncState.assert("f6", 1, 6));

		assertEquals("[f0,Retracted,f1,f3,f4,f5,f6]", itsSyncState.itsSlidingWindow.toString());
		
		temp = itsSyncState.retract("f3", 0, 6);
		
		assertNotSame(itsSyncState, temp);
		
		assertTrue(temp instanceof NoGaps);
		

	}


	// When a new device is first encountered, we do not know about any of the facts
	// it has previously asserted (and retracted). We are interested in gaining access
	// to the current state and are therefore interested to omit any facts that have
	// been asserted at some point and retracted later on. This is possible because of
	// the Sliding Window technique, which ensures that the facts which are currently 
	// true are in a consecutive list with no gaps.
	//
	// Note that as a consequence the order in the consecutive list is not necessarily 
	// the same order as the one in which the facts were asserted.
	private void firstEncounter_noRetractions() {
		int retractedFacts 	= 0;
		int totalAsserted	= 5;
		
		// When receiving a beat(0, T) from an unknown device, go to a state where some
		// asserted facts are missing. This is an AssertionGap state:
		itsSyncState = new AssertionGap(
				/* The sliding window */ 	new SlidingWindow(retractedFacts, totalAsserted),
				/* The device's ID */ 		null,
				/* No facts retracted */	retractedFacts,
				
				/* We need to find all asserted facts in the range [ retractedFacts, totalAsserted ] */
				/* Start of the gap */		retractedFacts,
				/* End of the gap */		totalAsserted
				);
	}
	
	private void firstEncounter_withRetractions() {
		int retractedFacts 	= 1;
		int totalAsserted	= 6;
		
		// When receiving a beat(0, T) from an unknown device, go to a state where some
		// asserted facts are missing. This is an AssertionGap state:
		itsSyncState = new AssertionGap(
				/* The sliding window */ 	new SlidingWindow(retractedFacts, totalAsserted),
				/* The device's ID */ 		null,
				/* No facts retracted */	retractedFacts,
				
				/* We need to find all asserted facts in the range [ retractedFacts, totalAsserted ] */
				/* Start of the gap */		retractedFacts,
				/* End of the gap */		totalAsserted
				);
	}
	
	private void firstEncounter_noOperations() {
		int retractedFacts 	= 0;
		int totalAsserted	= 0;
		
		// When receiving a beat(0, 0) from an unknown device, go to a state where no
		// messages have been missed thus far
		
		itsSyncState = new NoGaps(
				/* The sliding window */ 	new SlidingWindow(retractedFacts, totalAsserted),
				/* The device's ID */ 		null,
				/* No facts retracted */	retractedFacts,
				/* No facts asserted */		totalAsserted
				);

	}
	
	// Performs five assertions in the proper order and ensures that the class of the 
	// synchronisation state does not change (their is no transition in the state diagram)
	// until the last message has been asserted.
	private void assertions_inOrder(int retracted) {
		Class concreteClass = itsSyncState.getClass();
		Class resultingClass = (Class)ASSERTION_TRANSITION.get(concreteClass);
		
		// Sometimes there are no state transitions at the end, most notably to the NoGaps
		// and RetractionGap classes. In this case, the last update does not change the 
		// identity of the sync state.
		boolean lastStateEqual = (concreteClass == resultingClass);
		
		SynchronisationState updated;
		
		updated = itsSyncState.assert("f0", retracted, retracted + 0);
		verifySyncState(updated, true, concreteClass);

		updated = itsSyncState.assert("f1", retracted, retracted + 1);
		verifySyncState(updated, true, concreteClass);

		updated = itsSyncState.assert("f2", retracted, retracted + 2);
		verifySyncState(updated, true, concreteClass);

		updated = itsSyncState.assert("f3", retracted, retracted + 3);
		verifySyncState(updated, true, concreteClass);

		updated = itsSyncState.assert("f4", retracted, retracted + 4);
		verifySyncState(updated, lastStateEqual, resultingClass);

	}
	
	// Performs five assertions which arrive out of order. Tests whether the gap is
	// properly detected and whether it is resolved at the proper time. Finally it
	// checks whether the proper state transition is made after all messages were 
	// received.
	private void assertions_notOrdered(int retracted) {
		Class concreteClass = itsSyncState.getClass();
		Class afterGapClass = (Class)ASSERTION_GAP_TRANSITION.get(concreteClass);
		Class resultingClass = (Class)ASSERTION_TRANSITION.get(concreteClass);
		
		// Sometimes there are no state transitions upon the detection of an assertion
		// gap, most notably to the AssertionGap and DoubleGap classes. The update that 
		// produces the gap will therefore not affect the identity of the sync state
		// and neither will the update that fixes the gap.
		boolean gapStateEqual = (concreteClass == afterGapClass);

		// Sometimes there are no state transitions at the end, most notably to the NoGaps
		// and RetractionGap classes. In this case, the last update does not change the 
		// identity of the sync state.
		boolean lastStateEqual = (concreteClass == resultingClass);
		
		SynchronisationState updated;
		
		updated = itsSyncState.assert("f0", retracted, retracted + 0);
		verifySyncState(updated, true, 			concreteClass);

		updated = itsSyncState.assert("f2", retracted, retracted + 2);
		verifySyncState(updated, gapStateEqual, afterGapClass);

		updated = itsSyncState.assert("f3", retracted, retracted + 3);
		verifySyncState(updated, true, 			afterGapClass);

		updated = itsSyncState.assert("f1", retracted, retracted + 1);
		verifySyncState(updated, gapStateEqual, concreteClass);

		updated = itsSyncState.assert("f4", retracted, retracted + 4);
		verifySyncState(updated, lastStateEqual, resultingClass);

	}
	
	// Performs four assertions where there should have been five instead. This 
	// test allows checking whether the correct transitions are made when a gap
	// is observed in the assertions and it can be used to set up further checks. 
	private void assertions_missingMessage(int retracted) {
		Class concreteClass = itsSyncState.getClass();
		Class afterGapClass = (Class)ASSERTION_GAP_TRANSITION.get(concreteClass);
		
		// Sometimes there are no state transitions upon the detection of an assertion
		// gap, most notably to the AssertionGap and DoubleGap classes. The update that 
		// produces the gap will therefore not affect the identity of the sync state
		// and neither will the update that fixes the gap.
		boolean gapStateEqual = (concreteClass == afterGapClass);
		
		SynchronisationState updated;
		
		updated = itsSyncState.assert("f0", retracted, retracted + 0);
		verifySyncState(updated, true, 			concreteClass);

		updated = itsSyncState.assert("f2", retracted, retracted + 2);
		verifySyncState(updated, gapStateEqual, afterGapClass);

		updated = itsSyncState.assert("f3", retracted, retracted + 3);
		verifySyncState(updated, true, 			afterGapClass);

		updated = itsSyncState.assert("f4", retracted, retracted + 4);
		verifySyncState(updated, true,			afterGapClass);

	}
	
	private void assertions_missingRetractMessage(int retracted) {
		Class concreteClass = itsSyncState.getClass();
		Class afterRetractionGapClass = (Class)RETRACTION_GAP_TRANSITION.get(concreteClass);
		Class resultingClass = (Class)ASSERTION_TRANSITION.get(afterRetractionGapClass);

		boolean gapStateEqual = (concreteClass == afterRetractionGapClass);
		boolean lastStateEqual = (afterRetractionGapClass == resultingClass);

		SynchronisationState updated;
		
		updated = itsSyncState.assert("f0", retracted, 		retracted + 0);
		verifySyncState(updated, true, 			 concreteClass);

		updated = itsSyncState.assert("f1", retracted, 		retracted + 1);
		verifySyncState(updated, true, 			 concreteClass);

		updated = itsSyncState.assert("f2", retracted + 1, 	retracted + 2);
		verifySyncState(updated, gapStateEqual,	 afterRetractionGapClass);

		updated = itsSyncState.assert("f3", retracted + 1,	retracted + 3);
		verifySyncState(updated, true,			 afterRetractionGapClass);

		updated = itsSyncState.assert("f4", retracted + 1, 	retracted + 4);
		verifySyncState(updated, lastStateEqual, resultingClass);

	}
	
	private void assertions_missingRetractAndAssertMessage(int retracted) {
		Class concreteClass = itsSyncState.getClass();
		Class afterAssertionGapClass = (Class)ASSERTION_GAP_TRANSITION.get(concreteClass);

		boolean assertGapStateEqual = (concreteClass == afterAssertionGapClass);

		boolean doubleGapStateEqual = (afterAssertionGapClass == DoubleGap.class);

		SynchronisationState updated;
		
		updated = itsSyncState.assert("f0", retracted, 		retracted + 0);
		verifySyncState(updated, true, 						concreteClass);

		updated = itsSyncState.assert("f2", retracted,		retracted + 2);
		verifySyncState(updated, assertGapStateEqual,		afterAssertionGapClass);

		updated = itsSyncState.assert("f3", retracted + 1,	retracted + 3);
		verifySyncState(updated, doubleGapStateEqual,		DoubleGap.class);

		updated = itsSyncState.assert("f4", retracted + 1, 	retracted + 4);
		verifySyncState(updated, true, 						DoubleGap.class);

	}

	
	/**
	 * When a new client enters the network it emits a beat(0,0) message to all connected
	 * devices. These devices have not yet missed any devices from the said device and can
	 * thus deal with messages as they arrive.
	 */
	public void testNewClientScenario() {
		{
			// Test Scenario 1: All messages arrive, in the correct order
			firstEncounter_noOperations();
			
			assertions_inOrder(0);			
		}

		{
			// Test Scenario 2: All messages arrive, but they do not appear in the correct order
			firstEncounter_noOperations();
			
			assertions_notOrdered(0);			
		}

		{
			// Test Scenario 3: One message is missing
			firstEncounter_noOperations();
			
			assertions_missingMessage(0);			
		}
	}
	
	/**
	 * When a new device comes into contact with a device which has previously asserted 
	 * facts, it needs to request the missing facts from the device.
	 */
	public void testDetectDeviceScenario() {
		{
			// Test Scenario 1: All messages arrive, in the correct order
			firstEncounter_noRetractions();
			
			assertions_inOrder(0);
		}

		{
			// Test Scenario 2: All messages arrive, but they do not appear in the correct order
			firstEncounter_noRetractions();
			
			assertions_notOrdered(0);			
		}

		{
			// Test Scenario 3: One message is missing
			firstEncounter_noRetractions();
			
			assertions_missingMessage(0);			
		}	
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	public void testMissedRetractionMessage() {
		{
			// Test Scenario 1: All messages arrive, in the correct order
			firstEncounter_noOperations();
			
			assertions_missingRetractMessage(0);
			
			SynchronisationState updated = itsSyncState.retract("f1", 0, 5);
			
			verifySyncState(updated, false, NoGaps.class);
		}

		{
			// Test Scenario 2: All messages arrive, but they do not appear in the correct order
			firstEncounter_noRetractions();
			
			assertions_missingRetractMessage(0);

			SynchronisationState updated = itsSyncState.retract("f1", 0, 5);
			
			verifySyncState(updated, false, NoGaps.class);
		}

		{
			// Test Scenario 3: One message is missing
			firstEncounter_withRetractions();
			
			assertions_missingRetractAndAssertMessage(1);

			// The retracted message has not yet been asserted...
			SynchronisationState updated = itsSyncState.retract("f1", 1, 6);
			
			// Hence the state cannot be updated...
			verifySyncState(updated, true, DoubleGap.class);
			
			// Assert the missing fact which is also the one that has been retracted
			updated = itsSyncState.assert("f1", 2, 2);
			
			// There should not be a gap any more
			verifySyncState(updated, false, NoGaps.class);
		}	
	}
	
	/**
	 * When a new device comes into contact with a device which has previously asserted 
	 * facts, it needs to request the missing facts from the device. It the detected 
	 * device has previously retracted some facts, these can be ignored. As a response
	 * to requests from other devices, these retraction commands may be resent. This 
	 * should not affect the new device, as he can discard these messages.
	 */
	public void testIgnoreOldRetracts() {
		{
			// Test Scenario 1: All messages arrive, in the correct order
			firstEncounter_withRetractions();
			
			assertions_inOrder(1);
			
			String serializedState = itsSyncState.itsSlidingWindow.toString();
			
			// Perform  retract which is outdated and should therefore be discarded
			SynchronisationState updated = itsSyncState.retract("f1", 0, 6);
			
			// Ensure that the state has not changed
			assertEquals(serializedState, updated.itsSlidingWindow.toString());
		}

		{
			// Test Scenario 2: All messages arrive, but they do not appear in the correct order
			firstEncounter_withRetractions();
			
			assertions_notOrdered(1);			

			String serializedState = itsSyncState.itsSlidingWindow.toString();
			
			// Perform  retract which is outdated and should therefore be discarded
			SynchronisationState updated = itsSyncState.retract("f1", 0, 6);
			
			// Ensure that the state has not changed
			assertEquals(serializedState, updated.itsSlidingWindow.toString());
		}

		{
			// Test Scenario 3: One message is missing
			firstEncounter_withRetractions();
			
			assertions_missingMessage(1);			

			String serializedState = itsSyncState.itsSlidingWindow.toString();
			
			// Perform  retract which is outdated and should therefore be discarded
			SynchronisationState updated = itsSyncState.retract("f1", 0, 6);
			
			// Ensure that the state has not changed
			assertEquals(serializedState, updated.itsSlidingWindow.toString());
			
			// Use the updated state for further computation
			itsSyncState = updated;
			
			// The retracted message is also the one whose assertion is currently missing
			// Should the assertion arrive later on, the retraction should still not affect
			// the system, the expected state is simply a complete system
			updated = itsSyncState.assert("f1", 1, 2);
			
			verifySyncState(updated, false, NoGaps.class);
			
			assertEquals("[f0,f1,f2,f3,f4]", updated.itsSlidingWindow.toString());
			
		}	
	}
}
