package edu.vub.crime.parser;

import java.util.Iterator;

import edu.vub.crime.CrimeEngine;
import edu.vub.crime.grammar.Expression;
import edu.vub.crime.grammar.ExpressionList;
import edu.vub.crime.grammar.GenericFact;
import edu.vub.crime.grammar.GrammarVisitor;
import edu.vub.crime.grammar.Rule;
import edu.vub.crime.rete.agenda.UserActivation;

public class EvaluationVisitor extends GrammarVisitor {

	protected final CrimeEngine itsEngine;
	protected final boolean		shouldActivate;
	
	public EvaluationVisitor(CrimeEngine theEngine, boolean activateP) {
		super("evaluate", " directly");
		
		itsEngine 		= theEngine;
		shouldActivate	= activateP;
	}
	
	public void visit(Rule theNode) {
		if(shouldActivate) {
			itsEngine.deploy(theNode);
		} else {
			itsEngine.undeploy(theNode);
		}
	}
	
	public void visit(GenericFact theNode) {
		itsEngine.insertActivation(new UserActivation(theNode, shouldActivate));
	}
	
	public void visit(ExpressionList theNode) {
		for (Iterator exprIter = theNode.iterator(); exprIter.hasNext();) {
			Expression expr = (Expression) exprIter.next();
			expr.accept(this);
		}
	}
}
