package edu.vub.crime.parser;

import java.util.Iterator;

import edu.vub.crime.Assert;
import edu.vub.crime.AssertQualifiedFact;
import edu.vub.crime.grammar.Event;
import edu.vub.crime.grammar.Expression;
import edu.vub.crime.grammar.ExpressionList;
import edu.vub.crime.grammar.GenericFact;
import edu.vub.crime.grammar.GrammarVisitor;
import edu.vub.crime.grammar.attributes.datastructures.Pair;

/**
 * The RuleConsequenceVisitor visitor visits the lefthand side of rules to transform all
 * facts and qualified facts into the corresponding events. 
 *  
 * @author smostinc
 *
 */
public class RuleConsequenceVisitor extends GrammarVisitor {
	
	private final ExpressionList itsEvents;
	
	public RuleConsequenceVisitor() {
		super("transform", " into an event");
		itsEvents = new ExpressionList();
	}
	
	public ExpressionList getEvents() {
		return itsEvents;
	}
	
	public void visit(ExpressionList theNode) {
		for(Iterator itr = theNode.iterator(); itr.hasNext();) {			  			
  			((Expression)itr.next()).accept(this);
  		}
	}
	
	public void visit(Event theNode) {
		itsEvents.add(theNode);
	}
	
	public void visit(GenericFact theNode) {
		Event theEvent;
		
		if(theNode.isPublic()) {
			theEvent = new Event(new AssertQualifiedFact(), theNode.getLabel());
		} else {
			theEvent = new Event(new Assert(), theNode.getLabel());
		}
		
		for (Iterator fieldIt = theNode.getRepresentation().iterator(); fieldIt.hasNext();) {
			Pair currentField = (Pair) fieldIt.next();
			
			theEvent.addNamedAttribute(currentField.getName(), currentField.getValue());
		}
		
		itsEvents.add(theEvent);
	}
}
