package edu.vub.crime.rete.agenda;

import java.io.Serializable;
import java.util.Iterator;

import edu.vub.crime.CrimeEngine;
import edu.vub.crime.rete.data.Token;

/** <pre>
 * An activation stores actions that must be executed.
 * These actions are a consequences of the inserting of a certain token.
 * There are different kind of activations.
 * A first one is the RuleActivation, which is caused by the triggering of a rule.
 * Such a rule activation contains the actions, the left-hand-side (consequences)
 * of that triggered rule.
 * An other kind of activation is an ExternalActivation.
 * This activation is exchanged between colocated devices.
 * This exchange can take place because the two devices have just connect
 * and so their shared facts need to be exchanged.
 * (all those facts that need to be shared, are considered as several
 *  actions which are stored in this ExternalActivation)
 * Another possible reason for this kind of Activation is the triggering
 * of a rule on a colocated device, with an action (adding / removing)
 * of a non-private (so shared) fact as its consequence.
 * In this case, the shared facts needs to be updated in the fact spaces
 * of the colocated devices as well, which leads to an ExternalActivation.
 * </pre> */
abstract public class Activation implements Serializable {

	private Token itsToken;	// the token which has caused this activation
	
	public Activation(Token aToken) {
		itsToken = aToken;
	}
	
    /**
     * Returns the token which has caused this activation.
     */
	public Token token() {
		return itsToken;
	}
	
	
	abstract public void execute(CrimeEngine engine);
	
	abstract public boolean isComplementOf(Activation possibleComplement);
	
	abstract public Iterator iterator();
}
