package edu.vub.crime.rete.agenda;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import edu.vub.crime.CrimeEngine;
import edu.vub.crime.grammar.GenericFact;
import edu.vub.crime.rete.data.DataFactory;
import edu.vub.crime.rete.data.Token;
import edu.vub.crime.rete.data.TokenTag;

/** 
 * An ExternalActivation instance represents an activation which is received over the network layer. 
 * It contains a series of assertions and retractions of qualified facts. 
 */
public class ExternalActivation extends Activation {

	/**
	 * Maps the name of a particular space to all tokens that need to be processed by the engine.
	 */
	// TODO private final Map itsSpaceToTokenMap;
	private final Set itsSpaceToTokenMap;

	private Serializable itsPublisher;
	
    /** 
     * Constructor.
     */	
    public ExternalActivation() {
    	super(DataFactory.token(TokenTag.unconditional));
    	itsSpaceToTokenMap = new HashSet();
    }
    
    public ExternalActivation(int theTag, GenericFact theInput, Serializable thePublisher) {
    	this();
    	  	
    	itsSpaceToTokenMap.add(
    			theTag == TokenTag.remove?
    					DataFactory.removeToken(theInput):
    					DataFactory.addToken(theInput));
    	itsPublisher = thePublisher;
    }
    
    public Serializable getPublisher() {
    	return itsPublisher;
    }
    
    public void setPublisher(Serializable thePublisher) {
		itsPublisher = thePublisher;
	}
    
    /** 
     * Execute this activation, thereby asserting or retracting qualified facts 
     * in this engine. This assertion and retraction of facts needs not to be 
     * propagated any further.
     */
	public void execute(CrimeEngine engine) {
		for(Iterator entryIt = itsSpaceToTokenMap.iterator(); entryIt.hasNext(); ) {
			Token currentToken = (Token) entryIt.next();
			
			GenericFact theFact = currentToken.getFact(0);
			theFact.setPublisher(itsPublisher.toString());
			
			switch (currentToken.getTag()) {
			case TokenTag.add:
				engine.assert(theFact);
				break;
			case TokenTag.remove:
				engine.retract(theFact);
				break;
			}
        }
	}

	/**
	 * External activations are currently never considered to be a complement since
	 * the tokens in an external activation only contain a dummy token.
	 */
	public boolean isComplementOf(Activation possibleComplement) {
		return false;
	}
	
	public Iterator iterator() {
		return itsSpaceToTokenMap.iterator();
	}
}