package edu.vub.crime.rete.agenda;

import edu.vub.crime.CrimeEngine;


/** 
 * A scheduler executes the activations that are scheduled in the Rete engine's Agenda.
 * This particular scheduler provides applications with control over when activations
 * will be processed as it requires the application to call its {@link #step()} method.
 * 
 * @see ThreadedScheduler which starts its own thread and processes activations continuously.
 * 
 * @author smostinc
 */
public class InlineScheduler extends Scheduler {

	public InlineScheduler(Agenda theAgenda, CrimeEngine theEngine) {
		super(theAgenda, theEngine);
	}
	
	/**
	 * Executes a single activation if possible.
	 * 
	 * @return a boolean indicating whether the agenda contained an activation to execute.
	 */
	public boolean step() {
		Activation theNextActivation = itsAgenda.pollForActivation();
		
		if(theNextActivation != null) {
    		itsEngine.beginActivation(theNextActivation);
    		theNextActivation.execute(itsEngine);
    		itsEngine.endActivation(theNextActivation);
    		
    		return true;
		}
		
		return false;
	}
}
