package edu.vub.crime.rete.agenda;

import java.util.Iterator;

import edu.vub.crime.CrimeEngine;
import edu.vub.crime.grammar.Rule;
import edu.vub.crime.rete.data.Token;

/**
 * A RuleActivation is an activation which is the consequence of triggering a local rule.
 * The activation is supplied with the corresponding {@link Rule} for which the consequences
 * should be executed.
 * 
 * @author cfscholl
 * @author ephilips
 */
public class RuleActivation extends Activation {

    private final Rule 	itsRule;	// the rule which was satisfied by the given token
	
	/**
	 * Default Constructor.
     */
    public RuleActivation(Token theToken, Rule theRule) {
    	super(theToken);
    	
    	itsRule = theRule;
    }
    
	/**
	 * Returns the rule which was satisfied by the given token.
	 */
	public Rule rule() {
		return itsRule;
	}

	/**
	 * Executes the rule's consequence with the token which is stored in this activation.
	 */
	public void execute(CrimeEngine engine) {
		itsRule.execute(engine, token());
	}
	
	/**
	 * Allows determining whether this activation is the complement of the provided parameter.
	 */
	public boolean isComplementOf(Activation theActivation) {
		if(theActivation instanceof RuleActivation) {
			RuleActivation possibleComplement = (RuleActivation)theActivation;
			
			return 
				(itsRule == possibleComplement.rule()) &&
				(token().isComplementOf(possibleComplement.token()));
		} else {
			return false;
		}
	}
	
	public Iterator iterator() {
		throw new RuntimeException("Not supported");
	}
}
