package edu.vub.crime.rete.agenda;

import edu.vub.crime.CrimeEngine;


/** 
 * A scheduler executes the activations that are scheduled in the Rete engine's {@link Agenda}.
 * This particular scheduler starts its own thread and will thus process activations in
 * parallel to any concurrent processes.
 * 
 * @see InlineScheduler which allows programmer control over when activations are processed.
 * 
 * @author smostinc
 */
public class ThreadedScheduler extends Scheduler implements Runnable {

	Thread		itsExecutor;
	
	public ThreadedScheduler(Agenda theAgenda, CrimeEngine theEngine) {
		super(theAgenda, theEngine);
		
		itsExecutor = new Thread(this, "CrimeEngine -> Scheduler Thread");
		
		itsExecutor.start();
	}

	public void loopBody() {
   		try {
			Activation theNextActivation = itsAgenda.getNextActivation();
			
			itsEngine.beginActivation(theNextActivation);
			theNextActivation.execute(itsEngine);
			itsEngine.endActivation(theNextActivation);
		} catch (InterruptedException e) {
			return;
		}
	}
	
    public void run() {
    	while(itsAgenda.hasNext()) { loopBody(); }
    }
    
    public Thread getExecutor() {
    	return itsExecutor;
    }
}
