package edu.vub.crime.rete.agenda;

import java.util.Iterator;
import java.util.NoSuchElementException;

import edu.vub.crime.CrimeEngine;
import edu.vub.crime.grammar.GenericFact;
import edu.vub.crime.rete.data.DataFactory;
import edu.vub.crime.rete.data.TokenTag;

/**
 * The UserActivation class represents an activation which issued by the user either 
 * through the input file or through the read-eval-print loop.
 *  
 * @author smostinc
 */
public class UserActivation extends Activation {
	
	private final GenericFact	itsFact;
	private final boolean		shouldAssert;
	
	public UserActivation(GenericFact theFactToBeAssertedOrRetracted, boolean assertP) {
		super(DataFactory.token(TokenTag.unconditional));
		
		itsFact 		= theFactToBeAssertedOrRetracted;
		shouldAssert	= assertP;
	}
	
	public void execute(CrimeEngine engine) {
		if(shouldAssert)
			itsFact.assert(engine);
		else
			itsFact.retract(engine);
	}
	
	public boolean isComplementOf(Activation possibleComplement) {
		if(possibleComplement instanceof UserActivation) {
			UserActivation aPossibleComplement = (UserActivation) possibleComplement;
			
			return 
				(shouldAssert ^ aPossibleComplement.shouldAssert) 	// One asserts while the other retracts
				&& itsFact.equals(aPossibleComplement.itsFact);		// And both refer to the same fact.
		}
		return false;
	}
	
	public Iterator iterator() {
		return new Iterator() {
			public boolean hasServedAlready = false;
			
			public Object next() {
				if(! hasServedAlready) {
					hasServedAlready = true;
					return DataFactory.addToken(itsFact);
				}
				
				throw new NoSuchElementException();
			}
			
			public boolean hasNext() {
				return ! hasServedAlready;
			}
			
			public void remove() { }
		};
	}
}
