package edu.vub.crime.rete.combiner;

import java.util.Vector;

import edu.vub.crime.rete.data.MemoryTable;
import edu.vub.crime.rete.data.Token;

/** 
 * The Combiner class is used to join together two tokens, provided that a few constraints
 * are met. These constraints relate the bindings of variables in the left token to those
 * of the right token and vice versa.
 * <p>
 * Concrete subclasses of this class should provide combineLeft and combineRight methods 
 * which implement the combining of a newly inserted token with matching tokens from the 
 * memory when these constraints are met. The need for a disambiguation between combineLeft
 * and combineRight stems from the fact that tokens are ordered and thus there is a subtle
 * difference between receiving a token from the left (whose facts should proceed the ones
 * of tokens retrieved from the right memory) and receiving one from the right (whose facts
 * appear after those of the tokens retrieved from the left memory).
 * 
 * @author cfscholl
 * @author ephilips
 */
public abstract class Combiner {

	protected MemoryTable 	itsLeftMemory;
	protected MemoryTable 	itsRightMemory;
	
	protected Vector 		itsConstraints;
	
	public void setLeftMemory(MemoryTable theLeftMemory) {
		itsLeftMemory = theLeftMemory;
	}
	
	public void setRightMemory(MemoryTable theRightMemory) {
		itsRightMemory = theRightMemory;
	}
	
	public void setConstraints(Vector theConstraints) {
		itsConstraints = theConstraints;
	}
	
	public abstract Vector combineLeft(Token token);
	
	public abstract Vector combineRight(Token token);
}
