
package edu.vub.crime.rete.data;

import java.io.PrintStream;
import java.util.Iterator;
import java.util.Vector;

import edu.vub.crime.rete.filter.Join;
import edu.vub.crime.rete.nodes.JoinNode;

/** 
 * A MemoryTable is a container for {@link Token} instances. Each {@link JoinNode} has two 
 * such memory tables which store tokens it has previously received as input from both the
 * left- and the right-hand side. 
 * <p>
 * The MemoryTable does not provide any matching operations, it is simply a container class.
 * Note that the order in which tokens are stored is not specified.
 * 
 * @author cfscholl
 * @author ephilips
 */
public class MemoryTable {
	
	private Vector tokens_;

	/**
	 * Default Constructor. Creates an empty MemoryTable.
	 */
	public MemoryTable() {
		tokens_ = new Vector();
	}
		
	/**
	 * Insert a new token.
	 */
	public void insert(Token token) {
		tokens_.add(token);
	}
	
	/**
	 * Remove a certain token.
	 */
	public void remove(Token token) {
		tokens_.remove(token);
	}
    
    /** <pre>
     * Return the iterator of the datastructure.
     * </pre> */
    public Iterator iterator() {
    	return tokens_.iterator();
    }
    
    /**
     * Clear all tokens.
     */
    public void clear() {
    	tokens_.clear();
    }
 
    /** <pre>
     * Return the number of tokens.
     * </pre> */
    public int size() {
    	return tokens_.size();
    }
    
    public void print(PrintStream out) {
    	out.println();
    	out.println("MEMORYTABLE");
    	out.println("___________");
    	for(Iterator it=tokens_.iterator(); it.hasNext(); ) {
    		Token token = (Token) it.next();
    		out.println("token : " + token);
    	}
    	out.println();
    	out.println();
    }

	public void replace(Token token, Vector filters) {
		boolean inserted = false;
		Vector indices = new Vector();
		for(int idx=0; idx != tokens_.size(); idx++) {
			Token tkn = (Token) tokens_.elementAt(idx);
			boolean pass = true;
			for(Iterator fIt=filters.iterator(); fIt.hasNext(); ) {
				Join filter = (Join) fIt.next();
				pass = pass && filter.join(token, tkn);
			}
			if(pass) {
				indices.add(new Integer(idx));
				inserted = true;
			}
		}
		if(inserted) {
			int idx = 0;
			for(Iterator it=indices.iterator(); it.hasNext(); ) {
				Integer index = (Integer) it.next();
				tokens_.remove(index.intValue() - idx++);
			}
			tokens_.add(token);
		}
		else {
			tokens_.add(token);
		}
	}
	
	
	public String toString() {
		return tokens_.toString();
	}
}