package edu.vub.crime.rete.filter;

import edu.vub.crime.exceptions.IncorrectAttributeIndexException;
import edu.vub.crime.grammar.GenericFact;
import edu.vub.crime.grammar.attributes.Attribute;
import edu.vub.crime.grammar.attributes.datastructures.Operator;
import edu.vub.crime.rete.data.Token;

/** 
 * <pre>
 * This filter is used to check that the first fact of a token has
 * a Attribute the same as the one passed to it by the constructor.
 * Checking this is frequently returning operation in the filterNodes.
 * 
 * Important to know is that attributes hold there index they have 
 * in a fact. So in the fact myFact(one) the attribute one keeps a 
 * index 0 wich can be accesed with the method getIndex().
 * 
 * Knowing this the filterAttribute becomes trivial.
 * It takes the first fact of the token then asks the attribute of the token 
 * witch have the same index as the example Attribute and then compares these
 * two attributes. When they are the same the pass method returns true,
 * otherwise false. 
 * 
 * Example :
 * 
 *  FilterAttribute( genre with index 0 )  and < add,  likeMyMusic(genre, number) >
 *  passes but
 *  FilterAttribute(genre with index 0 )  and < add,  likeMyMusic(notGenre, number) >
 *  does not pass
 * 
 * </pre> 
 * @author cfscholl
 * @author ephilips
 * @author smostinc
 */
public class FilterAttribute extends Filter {

	private final int		itsFactIndex;
    private final Operator	itsFilterComparison;

    protected	  Attribute itsFilterAttribute;

    public FilterAttribute(Attribute filterAttribute) {
    	this(0, filterAttribute, Operator.EQL);
    }
    
    public FilterAttribute(Attribute filterAttribute, Operator filterComparison) {
        this(0, filterAttribute, filterComparison);
    }

    public FilterAttribute(int theFactIndex, Attribute filterAttribute, Operator filterComparison) {
        itsFactIndex 		= theFactIndex;
    	itsFilterAttribute 	= filterAttribute;
        itsFilterComparison = filterComparison;
    }
    
    /** <pre> 
     * Does a certain token passes or not.
     * 
     * @param token:  The token to check.
     * 
     * @return a boolean indicating that the token passed. 
     * </pre> */
    public boolean pass(Token token) {
		try {
	        GenericFact fact 				= token.getFact(itsFactIndex);
	        Attribute 	compareAttribute	= fact.getAttribute(itsFilterAttribute.getIndex().iterator());
	        
	        return itsFilterComparison.evaluate(compareAttribute, itsFilterAttribute);
		} catch (IncorrectAttributeIndexException e) {
			return false;
		}
    }
}