package edu.vub.crime.rete.filter;

import java.util.Iterator;
import java.util.Set;

import edu.vub.crime.CrimeEngine;
import edu.vub.crime.dataflow.DataFlowAttribute;
import edu.vub.crime.dataflow.FilterFlow;
import edu.vub.crime.grammar.Expression;
import edu.vub.crime.grammar.ExpressionList;
import edu.vub.crime.grammar.attributes.Attribute;
import edu.vub.crime.grammar.attributes.Variable;
import edu.vub.crime.grammar.attributes.datastructures.Operator;
import edu.vub.crime.rete.nodes.FilterNode;
import edu.vub.crime.rete.nodes.Node;

public class FilterFactory {
	
	public static FilterKind filterKind(String kind) {
		return new FilterKind(kind);
	}
	
	public static FilterAttribute filterAttribute(Attribute attr) {
		// TODO Document
		// attributes can be variables provided they are bound the only vars which will be passed
		// to this function are scoped ones as they are removed from the variables set of an expression
		// such that they will not be filtered by the CrimeEngine#addFilterAttributes() method.
		if (attr instanceof Variable) {
			Variable scoped = (Variable) attr;
			return new FilterAttribute(scoped.getValue());
		}
		return new FilterAttribute(attr);
	}
	
	public static FilterAttribute filterFlow(DataFlowAttribute flow, Operator filterComparison, FilterNode node, CrimeEngine engine) {
		return new FilterFlow(flow, filterComparison, node, engine);
	}
	
	public static FilterSpace filterSpace(String space) {
		return new FilterSpace(space);
	}
	
	/** <pre>
	 * Makes a filter that takes two tokens and checks the 
	 * variables from the left token with those of the rigth token
	 * 
	 * The code iterates over all variables from the right fact.
	 * For each variable it searches the first occurance of the same variable
	 * from the left Expressions, and makes a VarChecker for the match.
	 * 
	 * Example:
	 * leftFactOne(?SomeVar) and rigthFactOne(?someVar).
	 * joinFilter.pass(Token(leftFactOne(a)),Token(rightFactOne(a))) == True
	 * joinFilter.pass(Token(leftFactOne(a)),Token(rightFactOne(b))) == False
	 * </pre> */
	public static void joinFilter(ExpressionList lExpressions, Expression rFact, Node theNode) {
		joinFilter(lExpressions, rFact, 0, theNode);
	}
	
	private static void joinFilter(ExpressionList lExpressions, Expression rFact, int rIndex, Node theNode) {
		Set variables = rFact.variables();

		for (Iterator varItr = variables.iterator(); varItr.hasNext();) {
			Variable var = (Variable) varItr.next();
			int lIndex = 0;
			boolean go = true;
			for(Iterator lFactItr=lExpressions.iterator(); lFactItr.hasNext()&go; ) {			
				Expression lExpression = (Expression) lFactItr.next();				
				go = lExpression.compileJoinConstraint(var, lIndex, rIndex, theNode);		
				lIndex += lExpression.tokenSize();
			}
		}
	}
}