package edu.vub.crime.rete.filter;

import edu.vub.crime.grammar.GenericFact;
import edu.vub.crime.rete.data.Token;

/** <pre>
 * This filter is used to check that the first fact of a token has
 * a certain type.
 * Checking this is frequently returning operation in the filterNodes.
 *
 * It takes the first fact of the token then asks the type of this fact 
 * and then compares it with the type givin by the construction of the filter.
 * 
 * Example :
 * 
 *  FilterKind(likeMyMusic)  and < add,  likeMyMusic(genre, number) >
 *  passes but
 *  FilterKind(notLikeMyMusic)  and < add,  likeMyMusic(notGenre, number) >
 *  does not pass
 * 
 * </pre> */
public class FilterKind extends Filter {
	
	    /** <pre> 
	     * Datamembers.
	     * type: the type of the filter
	     * </pre> */
	    private String kind_;

	    /** <pre> 
	     * Constructor for FilterKind .
	     * 
	     * @param kind: the kind that must be equal
	     * 
	     * </pre> */
	    public FilterKind(String kind) {
	        this.kind_ = kind;
	    }

	    /** <pre> 
	     * This function checks if a token can be passed.
	     * 
	     * @param token: the token which must be checked
	     * 
	     * @return boolean
	     * </pre> */
	    public boolean pass(Token token) {
	    	GenericFact fact = token.getFact(0);
	        return fact.getLabel().equals(this.kind_);
	    }
}