package edu.vub.crime.rete.filter;

import edu.vub.crime.grammar.GenericFact;
import edu.vub.crime.rete.data.Token;


/** <pre>
 * This filter is used to check that the first fact of a token is in 
 * a certain factSpace.
 * Checking this is frequently returning operation in the filterNodes.
 *
 * It takes the first fact of the token and checks that it is in the 
 * right space.
 * 
 * Example :
 * 
 *  FilterSpace(private)  and < add, private -> likeMyMusic(genre, number) >
 *  passes but
 *  FilterSpace(public)  and < add,  private -> likeMyMusic(genre, number) >
 *  does not pass
 * 
 * </pre> */
public class FilterSpace extends Filter {
	private String space_;

	public FilterSpace(String space) {
		this.space_ = space;
	}
	
	public boolean pass(Token token) {
		GenericFact fact = token.getFact(0);
        return fact.getSpace().equals(space_);
	}
}
