package edu.vub.crime.rete.filter;

import junit.framework.TestCase;
import edu.vub.crime.grammar.ExpressionFactory;
import edu.vub.crime.grammar.GenericFact;
import edu.vub.crime.grammar.attributes.Attribute;
import edu.vub.crime.grammar.attributes.datastructures.Operator;
import edu.vub.crime.parser.IndexingVisitor;
import edu.vub.crime.rete.data.DataFactory;
import edu.vub.crime.rete.data.Token;

public class FilterTest extends TestCase {

	public void testFilterAttribute() {
		FilterAttribute	theAttributeFilter;
		
		{	// Build the filter
			GenericFact		theSurroundingFact	= ExpressionFactory.unqualifiedFact("test");
			Attribute		theIndexedAttribute	= ExpressionFactory.attribute(0);
			
			// To build a filter, the attribute needs to be indexed. This is done by the following visitor
			IndexingVisitor theAttributeIndexer	= new IndexingVisitor(theSurroundingFact);
			
			theAttributeIndexer.nextAttribute("attr", theIndexedAttribute);
		
			// Build a filter which requires that the "attr" field is greater than 0; 
							theAttributeFilter	= 
				new FilterAttribute(0, theIndexedAttribute, Operator.GTH);
		}
		
		{	// Build a matching token
			GenericFact		theMatchingFact		= ExpressionFactory.unqualifiedFact("test");
			Attribute		theIndexedAttribute	= ExpressionFactory.attribute(1);
			
			// To build a filter, the attribute needs to be indexed. This is done by the following visitor
			IndexingVisitor theAttributeIndexer	= new IndexingVisitor(theMatchingFact);
			
			theAttributeIndexer.nextAttribute("attr", theIndexedAttribute);
		
			// Verify that the filter allows matching tokens to pass
			Token			thePassedToken		= DataFactory.addToken(theMatchingFact);
			
			assertTrue(theAttributeFilter.pass(thePassedToken));
		}
		
		{	// Build a non-matching token
			GenericFact		theNonMatchingFact	= ExpressionFactory.unqualifiedFact("test");
			Attribute		theIndexedAttribute	= ExpressionFactory.attribute(0);
			
			// To build a filter, the attribute needs to be indexed. This is done by the following visitor
			IndexingVisitor theAttributeIndexer	= new IndexingVisitor(theNonMatchingFact);
			
			theAttributeIndexer.nextAttribute("attr", theIndexedAttribute);
		
			// Verify that the filter rejects non-matching tokens
			Token			thePassedToken		= DataFactory.addToken(theNonMatchingFact);
			
			assertFalse(theAttributeFilter.pass(thePassedToken));
		}
		
		{	// Build a malformed token
			GenericFact		theMalformedFact	= ExpressionFactory.unqualifiedFact("test");
			Attribute		theIndexedAttribute	= ExpressionFactory.attribute(1);
			
			// To build a filter, the attribute needs to be indexed. This is done by the following visitor
			IndexingVisitor theAttributeIndexer	= new IndexingVisitor(theMalformedFact);
			
			theAttributeIndexer.nextAttribute("badIdx", theIndexedAttribute);
		
			// Verify that the filter rejects malformed tokens rather than propagating an index exception
			Token			thePassedToken		= DataFactory.addToken(theMalformedFact);
			
			assertFalse(theAttributeFilter.pass(thePassedToken));
		}
	}
}
