package edu.vub.crime.rete.filter;

import java.util.Iterator;
import java.util.Vector;

import edu.vub.crime.grammar.Expression;
import edu.vub.crime.grammar.ExpressionList;
import edu.vub.crime.grammar.attributes.ConstantAttribute;
import edu.vub.crime.grammar.attributes.Attribute;
import edu.vub.crime.grammar.attributes.AttributeVisitor;
import edu.vub.crime.grammar.attributes.CompoundAttribute;
import edu.vub.crime.grammar.attributes.Variable;
import edu.vub.crime.grammar.attributes.datastructures.Operator;
import edu.vub.crime.rete.nodes.Node;

public class UnresolvedConstraint {
	
	private Variable 	itsConstrainedVariable;
	private Operator 	itsConstraint;
	private Attribute	itsExpectedValue;
	private int			itsFactIndex = -1;
	
	public UnresolvedConstraint(Variable theConstrainedVariable, Operator theConstraint, Attribute theExpectedValue) {
		itsConstrainedVariable	= theConstrainedVariable;
		itsConstraint			= theConstraint;
		itsExpectedValue		= theExpectedValue;
	}
	
	public Variable getConstrainedVariable() {
		return itsConstrainedVariable;
	}
	
	public void setFactIndex(int theFactIndex) {
		itsFactIndex = theFactIndex;
	}
	
	public boolean isFullySpecified(ExpressionList alreadyCompiledExpressions) {
		int theCurrentlyConsideredFact = 0;
		
		// verify whether the list of compiledExpressions contains the variable
		for (Iterator expressionIt = alreadyCompiledExpressions.iterator(); expressionIt.hasNext() & itsFactIndex < 0; theCurrentlyConsideredFact++) {
			Expression currentExpression = (Expression) expressionIt.next();
			
			for (Iterator variableIt = currentExpression.variables().iterator(); variableIt.hasNext();) {
				Variable currentVariable = (Variable) variableIt.next();
				
				if(currentVariable.equals(itsConstrainedVariable)) {
					itsConstrainedVariable.setIndex(currentVariable.getIndex());
					itsFactIndex = theCurrentlyConsideredFact;
					
					break;
				}
			}
		}
		
		boolean theResult = itsFactIndex >= 0;
		
		if (itsExpectedValue instanceof Variable) {
			Variable theVariable = (Variable) itsExpectedValue;
			
			theResult &= (theVariable.getValue() != null || alreadyCompiledExpressions.variables().contains(theVariable));
		} else if (itsExpectedValue instanceof CompoundAttribute) {
			CompoundAttribute theCompoundAttribute = (CompoundAttribute) itsExpectedValue;
			
			theResult &= alreadyCompiledExpressions.variables().containsAll(theCompoundAttribute.getVariables());
		}
		
		return theResult;
	}
	
	public void compile(final ExpressionList alreadyCompiledExpressions, final Node theNodeToExtendWithFilters) {
		itsExpectedValue.accept(new AttributeVisitor("resolve an inequality constraint for", "") {
			
			// The attributes of a constraint are properly indexed to allow dealing with compound
			// attributes in conditions.  However, the first ones are always arguments and 0,  as
			// the indexer is only asked to index one (possibly compound) attribute. This first 
			// index should be replaced by the index of the constrained variable, which is the 
			// starting point to identify the (sub)attribute.
			private Attribute reindexAttribute(Attribute theAttribute) {
				// First of all, clone the attribute to ensure that this function does not tamper
				// with its input attribute.
				Attribute theReindexedAttribute = (Attribute)theAttribute.clone();
				
				// Copy the index of the variable
				Vector theAmendedIndex = (Vector)itsConstrainedVariable.getIndex().clone();
				
				// Then iterate over the index of the attribute, but skip the first 2 entries (arguments, 0); 
				Iterator indexIt = itsConstrainedVariable.getIndex().iterator(); indexIt.next(); indexIt.next();
				
				for (; indexIt.hasNext();) {
					Object currentindex = indexIt.next();
					
					theAmendedIndex.add(currentindex);
				}
				
				theReindexedAttribute.setIndex(theAmendedIndex);
				
				return theReindexedAttribute;
			}
						
			public void visit(ConstantAttribute theSymbol) {
				Attribute theCorrectlyIndexedAttribute = reindexAttribute(theSymbol);
				
				theNodeToExtendWithFilters.addNewFilter(new FilterAttribute(itsFactIndex, theCorrectlyIndexedAttribute, itsConstraint));
			}
						
			public void visit(Variable theExpectedVariable) {
				int theCurrentlyConsideredFact = 0;
				
				for (Iterator expressionIt = alreadyCompiledExpressions.iterator(); expressionIt.hasNext(); theCurrentlyConsideredFact++) {
					Expression currentExpression = (Expression) expressionIt.next();
					
					for (Iterator variableIt = currentExpression.variables().iterator(); variableIt.hasNext();) {
						Variable currentVariable = (Variable) variableIt.next();
						
						if(currentVariable.equals(theExpectedVariable)) {
							theExpectedVariable.setIndex(currentVariable.getIndex());
							
							theNodeToExtendWithFilters.addNewFilter(
									new FilterVariableConstraint(
											itsFactIndex, theCurrentlyConsideredFact, 
											itsConstrainedVariable.getIndex(), theExpectedVariable.getIndex(), itsConstraint));
							
							return;
						}
					}
				}
			}

		});
	}
}
