/** <pre> 
 * @file FilterNode.java
 * @author Christophe Scholliers
 * @date 2006-09-23
 * </pre> */

package edu.vub.crime.rete.nodes;

import java.util.Iterator;

import edu.vub.crime.rete.data.Token;
import edu.vub.crime.rete.data.TokenTag;


/** 
 * A FilterNode is a ReteNode which takes but a single token as input. Such nodes are traditionally
 * called nodes of the ALPHA network. Filter nodes contain filters to verify whether the tokens have
 * acceptable values in their various attributes (i.e. they check for the presence of constants or
 * enforce certain relations). Filter nodes may also contain Joins to verify whether two occurrences
 * of a single variable within a pattern have been bound to the same value.
 */
public class FilterNode extends Node {

	public void reevaluate() {
		// Undo everything I've said in the past
		for (Iterator tokenIt = itsMemory.iterator(); tokenIt.hasNext();) {
			Token currentToken = (Token) tokenIt.next();
			Token negatedToken = new Token(TokenTag.remove);
			
			currentToken.copyFacts(negatedToken);
			
			propagate(negatedToken);
		}
		
		// Reevaluate all facts with the new constraints
		for (Iterator tokenIt = itsParent.itsMemory.iterator(); tokenIt.hasNext();) {
			Token currentToken = (Token) tokenIt.next();
			
			insert(currentToken);
		}
	}
	
	public void insert(Token theToken) {
		if(		// IF all constraints regarding duplicate variables hold AND
				join(theToken, theToken) &&
				// all constraints with respect to constant attributes hold
				pass(theToken)) {
			
			// THEN store the token in the memory and propagate it to all child nodes
			record(theToken);
			
			propagate(theToken);
		}
	}
}