package edu.vub.crime.rete.nodes;

import java.util.Iterator;
import java.util.Stack;

import edu.vub.crime.CrimeTestCase;
import edu.vub.crime.grammar.ExpressionList;
import edu.vub.crime.grammar.GenericFact;
import edu.vub.crime.grammar.attributes.datastructures.List;
import edu.vub.crime.parser.CrimeParser;
import edu.vub.crime.rete.CompileNetworkVisitor;
import edu.vub.crime.rete.data.Token;
import edu.vub.crime.rete.data.TokenTag;
import edu.vub.crime.rete.nodes.aux.TestNode;

public abstract class NodeTest extends CrimeTestCase {

	protected TestNode itsTestNode;

	public NodeTest() {
		super();
	}

	protected void setUp() throws Exception {
		super.setUp();
		
		itsTestNode = new TestNode();
	}

	private void testFilter(String theFact, int theTokenTag) {
		ExpressionList theList = CrimeParser.parse(theFact);
		
		itsTestNode.reset();
		
		for (Iterator expressionIt = theList.iterator(); expressionIt.hasNext();) {
			GenericFact currentFact = (GenericFact) expressionIt.next();
			
			switch (theTokenTag) {
			case TokenTag.add:
				itsEngine.assert(currentFact);
				break;

			case TokenTag.remove:
				itsEngine.retract(currentFact);
				break;
			}
		}
	}

	protected void testFilterPasses(String theFact) {
		testFilter(theFact, TokenTag.add);
		
		itsTestNode.assertTriggered(true);
	}

	protected void testFilterRejects(String theFact) {
		testFilter(theFact, TokenTag.add);
		
		itsTestNode.assertTriggered(false);
	}

	protected void testFilterAsserts(String theFact) {
		testFilter(theFact, TokenTag.add);
		
		itsTestNode.assertTriggered(TokenTag.add);
	}

	protected void testFilterRetracts(String theFact) {
		testFilter(theFact, TokenTag.add);
		
		itsTestNode.assertTriggered(TokenTag.remove);
	}

	protected void testFilterExpands(String theFact) throws Exception {
		testFilter(theFact, TokenTag.add);
		
		assertEquals(2, itsTestNode.itsTokens.size());
		
		Token retractPrevious = (Token)itsTestNode.itsTokens.get(0);
		Token assertNewValue = (Token)itsTestNode.itsTokens.get(1);
		
		GenericFact prevAccumulator = retractPrevious.getFact(retractPrevious.size() - 1);
		GenericFact newAccumulator  = assertNewValue.getFact(retractPrevious.size() - 1);
		
		Stack indexVector = new Stack();
		indexVector.push("accumulated");
		
		List oldValue = (List)prevAccumulator.getAttribute(indexVector.iterator());
		List newValue = (List)newAccumulator.getAttribute(indexVector.iterator());
		
		assertEquals(oldValue.length() + 1, newValue.length());
	}

	protected void testFilterShrinks(String theFact) throws Exception {
		ExpressionList theList = CrimeParser.parse(theFact);
		
		itsTestNode.reset();
		
		for (Iterator expressionIt = theList.iterator(); expressionIt.hasNext();) {
			GenericFact currentFact = (GenericFact) expressionIt.next();
			
			itsEngine.retract(currentFact);
		}
		
		assertEquals(2, itsTestNode.itsTokens.size());
		
		Token retractPrevious = (Token)itsTestNode.itsTokens.get(0);
		Token assertNewValue = (Token)itsTestNode.itsTokens.get(1);
		
		GenericFact prevAccumulator = retractPrevious.getFact(retractPrevious.size() - 1);
		GenericFact newAccumulator  = assertNewValue.getFact(retractPrevious.size() - 1);
		
		Stack indexVector = new Stack(); 
		indexVector.push("accumulated");
		
		List oldValue = (List)prevAccumulator.getAttribute(indexVector.iterator());
		List newValue = (List)newAccumulator.getAttribute(indexVector.iterator());
		
		assertEquals(oldValue.length() - 1, newValue.length());
	}
	
	public void testRemovalRejects(String theFact) {
		testFilter(theFact, TokenTag.remove);
		
		itsTestNode.assertTriggered(false);
	}
	
	public void testRemovalAsserts(String theFact) {
		testFilter(theFact, TokenTag.remove);
		
		itsTestNode.assertTriggered(TokenTag.add);
	}
	
	public void testRemovalRetracts(String theFact) {
		testFilter(theFact, TokenTag.remove);
		
		itsTestNode.assertTriggered(TokenTag.remove);
	}
	protected FilterNode compileFilterTestNetwork(GenericFact theCurrentlyCompilingFact) {		
		// Create a compiler visitor which will be used to compile the Fact into a filter node
		CompileNetworkVisitor theCompiler = new CompileNetworkVisitor(itsEngine, itsRootNode);
		
		// Let the compiler traverse the Fact
		theCurrentlyCompilingFact.accept(theCompiler);
		
		// The result is a new filter node which checks whether published facts have matching constants etc.
		FilterNode theNewFilter = (FilterNode) theCompiler.getCompiledNode();
				
		theNewFilter.addChild(itsTestNode);
		
		return theNewFilter;
	}
	
	protected void compileJoinTestNetwork(String thePrecondition) {
		ExpressionList theConditions = CrimeParser.parseConditions(thePrecondition);
		CompileNetworkVisitor theConditionCompiler = new CompileNetworkVisitor(itsEngine, itsRootNode);
		
		theConditions.accept(theConditionCompiler);
		
		Node theJoinNode = theConditionCompiler.getCompiledNode();
		
		theJoinNode.addChild(itsTestNode);	
	}
}