package edu.vub.crime.rete.nodes;

import java.util.Iterator;

import edu.vub.crime.rete.data.DataFactory;
import edu.vub.crime.rete.data.Token;
import edu.vub.crime.rete.filter.FilterFactory;

public class ViewNode extends RootNode {
	
	RootNode itsEnginesRootNode;

	public ViewNode(RootNode theEnginesRootNode) {
		itsEnginesRootNode = theEnginesRootNode;
	}

	public FilterNode getChild(String kind) {
    	FilterNode filter = (FilterNode) itsFilterNodes.get(kind);
    	if(filter == null) {
    		filter = NodesFactory.filterNode();
    		
    		filter.addNewFilter(FilterFactory.filterKind(kind));
    		
    		itsFilterNodes.put(kind, filter);
    		
    		if(itsEnginesRootNode != null) {
    			FilterNode parentFilter = itsEnginesRootNode.getChild(kind);
    		
    			parentFilter.addChild(this);

    			// To ensure that we do not really have a parent node
    			itsParent = null;
    		}
    	}
		
    	return filter;
	}
	
	public void restrict(String kind) {
		FilterNode theViewFilterNode = (FilterNode) itsFilterNodes.get(kind);
		
		if(theViewFilterNode == null) {
			// facts of this kind were not yet propagated to this view
			theViewFilterNode = NodesFactory.filterNode();
    		
			theViewFilterNode.addNewFilter(FilterFactory.filterKind(kind));
    		
    		itsFilterNodes.put(kind, theViewFilterNode);
		} else {
			// either all facts were propagated or another restriction exists
			FilterNode theRootFilterNode = itsEnginesRootNode.getChild(kind);
			
			if(theRootFilterNode.itsChildNodes.contains(this)) {
				// All facts were previously just propagated to this view
				theRootFilterNode.unhook(this);
				
				// retract all facts that were added this way
				for(Iterator tokenIt = theRootFilterNode.itsMemory.iterator(); tokenIt.hasNext(); ) {
					Token currentToken = (Token)tokenIt.next();
					Token negatedToken = DataFactory.removeToken();
					
					currentToken.copyFacts(negatedToken);
					
					this.insert(negatedToken);
				}
			} // else { Another restriction was in place. Nothing special needs to be done here	}
		}
		
		// To ensure that we do not really have a parent node
		itsParent = null;
	}
}
