package edu.vub.crime.util;

import java.io.IOException;
import java.util.Enumeration;
import java.util.Properties;

import edu.vub.at.util.logging.Logger;

/**
 * Auxiliary class used to more easily access CRIME loggers.
 * 
 * @author tvcutsem
 */
public final class Logging {

	/**
	 * Logs information regarding all the use of a non-networked CRIME engine.
	 */
	public static final Logger Engine_LOG = Logger.getInstance("crime.engine");
	
	/**
	 * Logs information published by the example applications shipped with a CRIME distribution.
	 */
	public static final Logger Examples_LOG = Logger.getInstance("crime.application.examples");

	/**
	 * Logs information regarding the network status of a networked CRIME engine.
	 */
	public static final Logger Network_LOG = Logger.getInstance("crime.network");
	
	/**
	 * Logs m2mi-specific information regarding the network status of a m2mi-based CRIME engine.
	 */
	public static final Logger M2MI_LOG = Logger.getInstance("crime.network.m2mi");
	
	/**
	 * Logs jgroups-specific information regarding the network status of a m2mi-based CRIME engine.
	 */
	public static final Logger JGroups_LOG = Logger.getInstance("crime.network.jgroups");

	static {
		// intialize the Logging API
	
		Properties props = new Properties();
		try {
			props.load(Logging.class.getResourceAsStream("logging.props"));
			Enumeration allProperties = props.propertyNames();
			// interpret each KEY=VALUE pair in the log file as
			// Logger(KEY).setPriority(VALUE)
			// where KEY = the name of a logger and VALUE = a priority level (e.g. WARN, DEBUG, ...)
			while (allProperties.hasMoreElements()) {
				String loggerName = (String) allProperties.nextElement();
				try {
					Logger.getInstance(loggerName).setPriority(props.getProperty(loggerName));
				} catch (IllegalArgumentException e) {
					System.err.println("Illegal priority level for logger " + loggerName + ": " + props.getProperty(loggerName));
				}
			}
		} catch (IOException e) {
			System.err.println("Could not read properties file: " + e.getMessage());
		}
	}
	
}

